--
-- Mod: FS22_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersAnimalScaling by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- 26-11-2021
-- Version 1.0.0.8 - Added xml file
-- Version 1.0.0.9 - Fixed a bug
-- Version 1.0.1.0 - no change
-- Version 1.0.1.1 - no change
-- Version 1.0.1.2 - no change
-- Version 1.0.1.3 - no change
-- Version 1.0.1.4 - adapted to work with maizePlus definitions of animal input/output
-- Version 1.0.1.5 - no change
-- Version 1.0.1.6 - no change
-- Version 1.0.1.7 - no change
-- Version 1.0.1.8 - no change
-- Version 1.0.1.9 - no change
-- Version 1.0.2.0 - no change
-- Version 1.0.2.1 - no change
-- Version 1.0.2.2 - no change
-- Version 1.0.2.3 - no change

-- Objective

		-- This script can redefine parameters in the following categories:

		-- animalSubType.buyPrice
		-- animalSubType.sellPrice
		-- animalSubType.transportPrice
		-- animalSubType.input.water
		-- animalSubType.input.food
		-- animalSubType.input.straw
		-- animalSubType.output.milk
		-- animalSubType.output.liquidManure
		-- animalSubType.output.manure
		-- animalSubType.output.pallets

		-- There are two types of parameters to define: (1) amplitude and (2) time.
		-- The amplitude defines the value of a given parameter, and the time defines, when a given
		-- parameter should have this value. Here is an example.

		-- RN.animalScaling.input.food["COW_HOLSTEIN"][1]					= {106,		1862};
		-- RN.animalScaling.input.food["COW_HOLSTEIN"]["time"]				= {0,	  	48};

		-- This set of values defines that a cow of this type will have a food intake of 106 L/month
		--  when 0 months old and 1862 L/month when 48 months old.

-- Issues addressed

	-- Seasonal variation
		-- In FS22 there is seasonal variation in environmental parameters, but not in animal parameters.
		-- Grazing animals will spend most of the time grazing in pastures during the grazing months.
		-- In these months a majority of feed comes from grazing, hence trough feed intake should be reduced.
		-- Similarly, the output rate of manure and liquid manure is reduced, as these are not collected
		-- in the pits.
		-- For cows, the milk output rate should be the same in winter season and in grazing season, but
		-- feed intake, bedding straw, manure and liquid manure output should be downscaled.
		--
		-- Here is an example, where trough feed for cows are reduced to 10% during grazing, while
		-- feed intake for pigs are the same:
		--
		-- RN.animalSummerScaling.input.food["COW_SWISS_BROWN"] 				= 0.1;
		-- RN.animalSummerScaling.input.food["COW_HOLSTEIN"] 					= 0.1;
		-- RN.animalSummerScaling.input.food["COW_ANGUS"] 						= 0.1;
		-- RN.animalSummerScaling.input.food["COW_LIMOUSIN"] 					= 0.1;
		-- RN.animalSummerScaling.input.food["PIG_LANDRACE"] 					= 1;
		-- RN.animalSummerScaling.input.food["PIG_BLACK_PIED"] 					= 1;
		-- RN.animalSummerScaling.input.food["PIG_BERKSHIRE"] 					= 1;


		-- For sheep, there is feed intake and wool production to consider. In grazing season, the feed input
		-- should be grass by a very reduced amount, as most of it is obtained from grazing. In the winter
		-- season, the feed intake should be hay. Hay has a much higher dry matter content, hence the feed volume
		-- per animal per day is less than for fresh grass.
		-- The wool production should also be low in winter and high in the grazing season.

	-- Cow milk issue
		-- In the vanilla game, cows produce milk from they are 12 month old. In real life, a heifer
		-- do not produce milk until after the first calving. This occurs when the heifer is 29 month old.
		-- Here is how this is set (do not change it here, this is just a comment, change it down in the code):

		--	RN.animalScaling.output.milk["COW_HOLSTEIN"][1]					= {0,		0,		870};	-- milk liters/month
		--  RN.animalScaling.output.milk["COW_HOLSTEIN"]["time"]			= {0,	  	28,		 29};	-- lactation month

	-- Pig litter issue
		-- In FS22, a birthing sow produce ONE pig every four months. The gestation period of four months
		-- is realistic. The number of piglets is not. This will result in a very poor economy for hog farming.
		-- I have not found a way to change the litter size, and the time of birth is always at the start of
		-- the month, irrespective of the number of game days per month.
		--
		-- In real life, one mature landrace sow delivers 12 - 14 piglets in one farrowing. We cannot recreate
		-- this in a realistic way in FS22.
		--
		-- The easy way around this is not to grow your own piglet. If you want to have pigs on your farm,
		-- let it be a hog farm, fattening piglets for slaugther.
		-- Buy new piglets (0 month old) on the first day of a new month and sell them again when 5 or 6 month old.
		-- So, each time you bring bacon pigs to the animal dealer, bring back a load of piglets. It is that easy.
		--
		-- Let the pig capacity of your favorit animal transport wagon decide how many piglets to buy each month.
		-- If it is the small one taking 9 pigs, then buy 9 pigs evey month. You will then need a pen capacity of
		-- 6 months * 9 pigs/month = 54 pigs.

		-- With the default prices, a piglet costs 50 eu. After 6 month, you sell it for about 200 eu.
		-- The net value increase is 150 eu per pig over 6 month.

	-- Sheep issues
		-- There are no serious issues with sheep, as far as I know. But remember that the winter feed-intake rate
		-- used in this mod is based on hay. So feed grass in the grazing period and hay in the non-grazing period.

-- How to use the script

	-- If you want to change a value or the time of that value, simply edit the numbers in the script.
	-- Here is an example where I have redefined the age of lactation to be 29 months, that is the time
	-- when heifers do their first calving.
	-- I have also set the milk yield per cow to 29 liters/day, or 870 Liters/month.

	-- RN.animalScaling.output.milk["COW_HOLSTEIN"][1]				= {0,		0,		870};	-- milk liters/month
	-- RN.animalScaling.output.milk["COW_HOLSTEIN"]["time"]			= {0,	  	28,		 29};	-- lactation month

	-- All parameters called RN.animalSummerScaling.something scales the non-grazing value to a new value to be
	-- used in the grazing season.

	-- Example: RN.animalSummerScaling.input.food["COW_SWISS_BROWN"] 			= 0.1;

	-- This reduces the trough feed intake to 10% during the grazing season.

	-- Some parameters are left unscaled. Their scaling parameter is set to ONE.
	-- Some parameters are not in effect (e.g. milk production for horses). In this case the scaling parameter is set to ZERO.

	-- You can turn on/off whether the script should print to the log file during game load.

	-- local printToLog = true; -- set to false is no output should be printed to log file.

	-- While playing, you can print data to the log file by pressing lAlt-ra ("a" for animal). (hold down left-Alt while pressing r and a)
	-- The seasonal scaling between non-grazing and grazing periods is run in an update script every 10 min.
	-- Hence, a time delay of up to 10 min is experienced before new parameter values are in effect.

-- Strange observations

	-- A full slurry pit will reduce milk yield. So empty the slurry pit before it becomes full.



RealNumbersAnimalScaling = {};

function round(x)
	local f = math.floor(x)
	if (x == f) or (x % 2.0 == 0.5) then
		return f;
	else
		return math.floor(x + 0.5);
	end
end

function RealNumbersAnimalScaling:loadMap(name)
	local currency = "€";	-- or  "$" or "£"
	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;
	local FirstGrazingMonth = 5;
	local LastGrazingMonth  = 10;
	local printToLog 	= true;		-- set to false is no output should be printed to log file.
	local isDebug 		= false;

	RN.notRunYetRealNumbersAnimalScaling = true;
	RN.upStartRealNumbersAnimalScaling = true;



	print("\n --- RealNumbersAnimalScaling.lua started : press lAlt-ra --- \n");

-- DebugUtil.printTableRecursively(g_currentMission.animalSystem.subTypes,"_",0,5);

	local modSettingsDir = getUserProfileAppPath() .. "modSettings";
	if modSettingsDir == nil then
		print(" ");
		print("modSettingsDir undefined");
		print(" ");
	end;



	RN.animalScaling 												= {};
	RN.animalScaling.animalTypes									= {};
	RN.animalScaling.animalSubTypes									= {};

	print("\n --- Animal types --- \n");
	for i, animalType in pairs(g_currentMission.animalSystem.types) do
		RN.animalScaling.animalTypes[i] = {};
		local animalTypeNameMAP = string.upper(tostring(animalType.name));
		RN.animalScaling.animalTypes[i].typeNameMAP = animalTypeNameMAP;
		RN.animalScaling.numAnimalTypesMAP = i;
		print(i.." : "..animalTypeNameMAP);
	end;
	print("\n --- Animal subtypes --- \n");
	for i, animalSubType in pairs(g_currentMission.animalSystem.subTypes) do

		local animalSubTypeSubTypeNameMAP = string.upper(tostring(animalSubType.name));
		local animalSubTypeTypeIndexMAP = animalSubType.typeIndex;
		local animalSubTypeTypeNameMAP = string.upper(g_currentMission.animalSystem.typeIndexToName[animalSubTypeTypeIndexMAP]);
		local animalSubTypeSubTypeIndexMAP = animalSubType.subTypeIndex;
		RN.animalScaling.animalSubTypes[animalSubTypeSubTypeIndexMAP] = {};
		RN.animalScaling.animalSubTypes[animalSubTypeSubTypeIndexMAP].subTypeSubTypeNameMAP = animalSubTypeSubTypeNameMAP;
		RN.animalScaling.animalSubTypes[animalSubTypeSubTypeIndexMAP].subTypeTypeIndexMAP = animalSubTypeTypeIndexMAP;
		RN.animalScaling.animalSubTypes[animalSubTypeSubTypeIndexMAP].subTypeTypeNameMAP = animalSubTypeTypeNameMAP;
		RN.animalScaling.animalSubTypes[animalSubTypeSubTypeIndexMAP].subTypeSubTypeIndexMAP = animalSubTypeSubTypeIndexMAP;
		RN.animalScaling.numAnimalSubTypesMAP = i;
		print(animalSubTypeSubTypeIndexMAP.." : "..animalSubTypeSubTypeNameMAP.." ; "..animalSubTypeTypeIndexMAP.." = "..animalSubTypeTypeNameMAP);
		if (animalSubType.buyPrice ~= nil) then            print("buyPrice keyframes      : "..animalSubType.buyPrice.numKeyframes); end;
		if (animalSubType.sellPrice ~= nil) then           print("sellPrice keyframes     : "..animalSubType.sellPrice.numKeyframes); end;
		if (animalSubType.transportPrice ~= nil) then      print("tranportPrice keyframes : "..animalSubType.transportPrice.numKeyframes); end;
		if (animalSubType.input.straw ~= nil) then         print("straw keyframes         : "..animalSubType.input.straw.numKeyframes); end;
		if (animalSubType.input.water ~= nil) then         print("water keyframes         : "..animalSubType.input.water.numKeyframes); end;
		if (animalSubType.input.food ~= nil) then          print("food keyframes          : "..animalSubType.input.food.numKeyframes); end;
		if (animalSubType.output.milk ~= nil) then         print("milk keyframes          : "..animalSubType.output.milk.numKeyframes); end;
		if (animalSubType.output.manure ~= nil) then       print("manure keyframes        : "..animalSubType.output.manure.numKeyframes); end;
		if (animalSubType.output.liquidManure ~= nil) then print("liquidManure keyframes  : "..animalSubType.output.liquidManure.numKeyframes); end;
		if (animalSubType.output.pallets ~= nil) then      print("pallet keyframes        : "..animalSubType.output.pallets.numKeyframes); end;
		print(" ");
	end;

	local savegameIndex = 0;
	if g_currentMission.missionInfo.savegameIndex ~= nil then
		savegameIndex = g_currentMission.missionInfo.savegameIndex;
	end;
	local RNmodSettingsDir = modSettingsDir .. "/RealLifeNumbers/savegame" .. savegameIndex;

	local myAnimalTypesXML = RNmodSettingsDir.."/my_animalTypes.xml";

--	RN.animalScaling.FirstGrazingMonth								= 5;		-- Grazing starts May 1st
--	RN.animalScaling.LastGrazingMonth								= 10;		-- Grazing ends October 31st
--	RN.animalScaling.isGrazingPeriod 								= false; 	-- Status parameter, do not change
	RN.animalScaling.currentMonth 									= (g_currentMission.environment.currentPeriod + 2) % 12;
	if RN.animalScaling.currentMonth == 0 then RN.animalScaling.currentMonth = 12; end;
	RN.animalScaling.pigReproductionMonth							= 4;		-- Vanilla is 4, but then you get one pig per sow in 4 months


	local currentMonth = (g_currentMission.environment.currentPeriod + 2) % 12; 	-- March is apparently month 1
	if currentMonth == 0 then currentMonth = 12; end;
--	FirstGrazingMonth = RN.animalScaling.FirstGrazingMonth;
--	LastGrazingMonth  = RN.animalScaling.LastGrazingMonth;

	RN.animalScaling.buyPrice										= {};
	RN.animalScaling.sellPrice										= {};
	RN.animalScaling.transportPrice									= {};
	RN.animalScaling.input											= {};
	RN.animalScaling.input.water									= {};
	RN.animalScaling.input.food										= {};
	RN.animalScaling.input.straw									= {};
	RN.animalScaling.output 										= {};
	RN.animalScaling.output.milk									= {};
	RN.animalScaling.output.liquidManure							= {};
	RN.animalScaling.output.manure									= {};
	RN.animalScaling.output.pallets 								= {};
	RN.animalScaling.FirstGrazingMonth								= {};
	RN.animalScaling.LastGrazingMonth								= {};

	RN.animalScaling.typeIndex										= {};
	RN.animalScaling.subTypeIndex									= {};
	RN.animalScaling.animalName										= {};

	RN.animalSummerScaling 											= {};
	RN.animalSummerScaling.buyPrice									= {};
	RN.animalSummerScaling.sellPrice								= {};
	RN.animalSummerScaling.transportPrice							= {};
	RN.animalSummerScaling.input									= {};
	RN.animalSummerScaling.input.water								= {};
	RN.animalSummerScaling.input.food								= {};
	RN.animalSummerScaling.input.straw								= {};
	RN.animalSummerScaling.output									= {};
	RN.animalSummerScaling.output.milk								= {};
	RN.animalSummerScaling.output.liquidManure						= {};
	RN.animalSummerScaling.output.manure							= {};
	RN.animalSummerScaling.output.pallets							= {};


	if fileExists(myAnimalTypesXML) then

		print("\n --- xml file exist for this savegame ---\n");
		print("\n --- data read from xml file ---\n");

		local xmlFile = loadXMLFile("animals", myAnimalTypesXML);
		if xmlFile ~= nil then
			print("Loading file:");
			print(myAnimalTypesXML);
			print(" ");
			local i = 0;
			while true do
				local xmlKeyType = string.format("animals.types.type(%d)", i);
				if not hasXMLProperty(xmlFile, xmlKeyType) then
					if isDebug then print(xmlKeyType.." end of list or does not have xml property"); end;
					break;
				end;
				local typeIndexXML = getXMLInt(xmlFile, xmlKeyType .. "#typeIndex");
				local animalTypeNameXML = string.upper(getXMLString(xmlFile, xmlKeyType .. "#name"));
				RN.animalScaling.FirstGrazingMonth[animalTypeNameXML] = tonumber(getXMLString(xmlFile, xmlKeyType .. "#FirstGrazingMonth"));
				RN.animalScaling.LastGrazingMonth[animalTypeNameXML] = tonumber(getXMLString(xmlFile, xmlKeyType .. "#LastGrazingMonth"));
				local firstGrazingMonth = RN.animalScaling.FirstGrazingMonth[animalTypeNameXML];
				local lastGrazingMonth = RN.animalScaling.LastGrazingMonth[animalTypeNameXML];
				if isDebug then print(animalTypeNameXML.." firstGrazingMonth = "..firstGrazingMonth.."  lastGrazingMonth = "..lastGrazingMonth);end;
				j = 0;
				while true do
					local xmlKeySubType = string.format(xmlKeyType..".subTypes.subType(%d)", j);
					if not hasXMLProperty(xmlFile, xmlKeySubType) then
					if isDebug then print(xmlKeySubType.." end of list or does not have xml property"); end;
						break;
					end;
					local animalSubTypeNameXML 	= string.upper(getXMLString(xmlFile, xmlKeySubType .. "#name"));
					local animalSubTypeIndexXML = getXMLInt(xmlFile, xmlKeySubType .. "#subTypeIndex");

-- get corresponding Map data

					local subTypeSubTypeNameMAP 	= RN.animalScaling.animalSubTypes[animalSubTypeIndexXML].subTypeSubTypeNameMAP;
					local subTypeTypeIndexMAP 		= RN.animalScaling.animalSubTypes[animalSubTypeIndexXML].subTypeTypeIndexMAP;
					local subTypeTypeNameMAP 		= RN.animalScaling.animalSubTypes[animalSubTypeIndexXML].subTypeTypeNameMAP;
					local subTypeSubTypeIndexMAP 	= RN.animalScaling.animalSubTypes[animalSubTypeIndexXML].subTypeSubTypeIndexMAP;

-- compare XML and Map data

					if animalSubTypeNameXML ~= subTypeSubTypeNameMAP then
						print(" ");
						print("Map uses subTypeName : ".. subTypeSubTypeNameMAP);
						print("XML uses subTypeName : ".. animalSubTypeNameXML);
						print("XML subTypeName overwritten by Map subTypeName");
						print("Match XML subTypeNames with Map subTypeNames in my_animalTypes.xml");
						print(" ");
						animalSubTypeNameXML = subTypeSubTypeNameMAP;		-- use map subType names
					end;
					if animalSubTypeIndexXML ~= subTypeSubTypeIndexMAP then
						print(" ");
						print("Map subTypeIndex : ".. subTypeSubTypeIndexMAP);
						print("XML subTypeIndex : ".. animalSubTypeIndexXML);
						print("Match XML subTypeIndex with Map subTypeIndex in my_animalTypes.xml");
						print(" ");
					end;
					if typeIndexXML ~= subTypeTypeIndexMAP then
						print(" ");
						print("Map typeIndex : ".. subTypeTypeIndexMAP);
						print("XML typeIndex : ".. typeIndexXML);
						print("Match XML typeIndex with Map typeIndex in my_animalTypes.xml");
						print(" ");
					end;
					if animalTypeNameXML ~= subTypeTypeNameMAP then
						print(" ");
						print("Map typeName : ".. subTypeTypeNameMAP);
						print("XML typeName : ".. animalTypeNameXML);
						print("Match XML typeName with Map typeName in my_animalTypes.xml");
						print(" ");
					end;

					if isDebug then print("animalSubTypeNameXML : "..animalSubTypeNameXML);end;

					local animalSubType = {}
					if g_currentMission.animalSystem.nameToSubType[animalSubTypeNameXML] ~= nil then
						animalSubType = g_currentMission.animalSystem.nameToSubType[animalSubTypeNameXML];
					end;


					RN.animalScaling.buyPrice[animalSubTypeNameXML] 										= {};
					RN.animalScaling.buyPrice[animalSubTypeNameXML][1]										= {};
					RN.animalScaling.buyPrice[animalSubTypeNameXML]["time"]									= {};
					RN.animalSummerScaling.buyPrice[animalSubTypeNameXML]									= {};

					RN.animalScaling.sellPrice[animalSubTypeNameXML]		 								= {};
					RN.animalScaling.sellPrice[animalSubTypeNameXML][1]										= {};
					RN.animalScaling.sellPrice[animalSubTypeNameXML]["time"]								= {};
					RN.animalSummerScaling.sellPrice[animalSubTypeNameXML]									= {};

					RN.animalScaling.transportPrice[animalSubTypeNameXML] 									= {};
					RN.animalScaling.transportPrice[animalSubTypeNameXML][1]								= {};
					RN.animalScaling.transportPrice[animalSubTypeNameXML]["time"]							= {};
					RN.animalSummerScaling.transportPrice[animalSubTypeNameXML]								= {};

					RN.animalScaling.input.water[animalSubTypeNameXML]										= {};
					RN.animalScaling.input.water[animalSubTypeNameXML][1]									= {};
					RN.animalScaling.input.water[animalSubTypeNameXML]["time"]								= {};
					RN.animalSummerScaling.input.water[animalSubTypeNameXML]								= {};

					RN.animalScaling.input.food[animalSubTypeNameXML]										= {};
					RN.animalScaling.input.food[animalSubTypeNameXML][1]									= {};
					RN.animalScaling.input.food[animalSubTypeNameXML]["time"]								= {};
					RN.animalSummerScaling.input.food[animalSubTypeNameXML]									= {};

					RN.animalScaling.input.straw[animalSubTypeNameXML]										= {};
					RN.animalScaling.input.straw[animalSubTypeNameXML][1]									= {};
					RN.animalScaling.input.straw[animalSubTypeNameXML]["time"]								= {};
					RN.animalSummerScaling.input.straw[animalSubTypeNameXML]								= {};

					RN.animalScaling.output.milk[animalSubTypeNameXML]										= {};
					RN.animalScaling.output.milk[animalSubTypeNameXML][1]									= {};
					RN.animalScaling.output.milk[animalSubTypeNameXML]["time"]								= {};
					RN.animalSummerScaling.output.milk[animalSubTypeNameXML]								= {};

					RN.animalScaling.output.liquidManure[animalSubTypeNameXML]								= {};
					RN.animalScaling.output.liquidManure[animalSubTypeNameXML][1]							= {};
					RN.animalScaling.output.liquidManure[animalSubTypeNameXML]["time"]						= {};
					RN.animalSummerScaling.output.liquidManure[animalSubTypeNameXML]						= {};

					RN.animalScaling.output.manure[animalSubTypeNameXML]									= {};
					RN.animalScaling.output.manure[animalSubTypeNameXML][1]									= {};
					RN.animalScaling.output.manure[animalSubTypeNameXML]["time"]							= {};
					RN.animalSummerScaling.output.manure[animalSubTypeNameXML]								= {};

					RN.animalScaling.output.pallets[animalSubTypeNameXML]									= {};
					RN.animalScaling.output.pallets[animalSubTypeNameXML][1]								= {};
					RN.animalScaling.output.pallets[animalSubTypeNameXML]["time"]							= {};
					RN.animalSummerScaling.output.pallets[animalSubTypeNameXML]								= {};

-- buyPrice

					k = 0;
					while true do
						local xmlKeyBuyPrice = string.format(xmlKeySubType..".buyPrices.buyPrice(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyBuyPrice) then
						if isDebug then print(xmlKeyBuyPrice.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyBuyPrice .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyBuyPrice .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyBuyPrice .. "#price");
						local price = getXMLFloat(xmlFile, xmlKeyBuyPrice .. "#price");
--						if isDebug then print(price); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyBuyPrice .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.buyPrice[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.buyPrice[animalSubTypeNameXML][1][k] 			= price;
						RN.animalSummerScaling.buyPrice[animalSubTypeNameXML][k] 		= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." price = "..price.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.buyPrice ~= nil then
--						animalSubType.buyPrice.numKeyframes = k;
					end;
-- sellPrice

					k = 0;
					while true do
						local xmlKeySellPrice = string.format(xmlKeySubType..".sellPrices.sellPrice(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeySellPrice) then
						if isDebug then print(xmlKeySellPrice.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeySellPrice .. "#age");
						local age = getXMLInt(xmlFile, xmlKeySellPrice .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeySellPrice .. "#price");
						local price = getXMLFloat(xmlFile, xmlKeySellPrice .. "#price");
--						if isDebug then print(price); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeySellPrice .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.sellPrice[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.sellPrice[animalSubTypeNameXML][1][k] 		= price;
						RN.animalSummerScaling.sellPrice[animalSubTypeNameXML][k] 		= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." price = "..price.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.sellPrice ~= nil then
--						animalSubType.sellPrice.numKeyframes = k;
					end;

-- transportPrice

					k = 0;
					while true do
						local xmlKeyTransportPrice = string.format(xmlKeySubType..".transportPrices.transportPrice(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyTransportPrice) then
						if isDebug then print(xmlKeyTransportPrice.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyTransportPrice .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyTransportPrice .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyTransportPrice .. "#price");
						local price = getXMLFloat(xmlFile, xmlKeyTransportPrice .. "#price");
--						if isDebug then print(price); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyTransportPrice .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.transportPrice[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.transportPrice[animalSubTypeNameXML][1][k] 		= price;
						RN.animalSummerScaling.transportPrice[animalSubTypeNameXML][k] 	= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." price = "..price.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.transportPrice ~= nil then
--						animalSubType.transportPrice.numKeyframes = k;
					end;
-- waterInput

					k = 0;
					while true do
						local xmlKeyWaterInput = string.format(xmlKeySubType..".waterInputs.waterInput(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyWaterInput) then
						if isDebug then print(xmlKeyWaterInput.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyWaterInput .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyWaterInput .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyWaterInput .. "#litersPerMonth");
						local litersPerMonth = getXMLFloat(xmlFile, xmlKeyWaterInput .. "#litersPerMonth");
--						if isDebug then print(litersPerMonth); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyWaterInput .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.input.water[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.input.water[animalSubTypeNameXML][1][k] 		= litersPerMonth;
						RN.animalSummerScaling.input.water[animalSubTypeNameXML][k] 	= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." litersPerMonth = "..litersPerMonth.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.input.water ~= nil then
--						animalSubType.input.water.numKeyframes = k;
					end;

-- foodInput

					k = 0;
					while true do
						local xmlKeyFoodInput = string.format(xmlKeySubType..".foodInputs.foodInput(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyFoodInput) then
						if isDebug then print(xmlKeyFoodInput.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyFoodInput .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyFoodInput .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyFoodInput .. "#litersPerMonth");
						local litersPerMonth = getXMLFloat(xmlFile, xmlKeyFoodInput .. "#litersPerMonth");
--						if isDebug then print(litersPerMonth); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyFoodInput .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.input.food[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.input.food[animalSubTypeNameXML][1][k] 		= litersPerMonth;
						RN.animalSummerScaling.input.food[animalSubTypeNameXML][k] 	= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." litersPerMonth = "..litersPerMonth.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.input.food ~= nil then
--						animalSubType.input.food.numKeyframes = k;
					end;

-- strawInput

					k = 0;
					while true do
						local xmlKeyStrawInput = string.format(xmlKeySubType..".strawInputs.strawInput(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyStrawInput) then
						if isDebug then print(xmlKeyStrawInput.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyStrawInput .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyStrawInput .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyStrawInput .. "#litersPerMonth");
						local litersPerMonth = getXMLFloat(xmlFile, xmlKeyStrawInput .. "#litersPerMonth");
--						if isDebug then print(litersPerMonth); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyStrawInput .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.input.straw[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.input.straw[animalSubTypeNameXML][1][k] 		= litersPerMonth;
						RN.animalSummerScaling.input.straw[animalSubTypeNameXML][k] 	= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." litersPerMonth = "..litersPerMonth.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.input.straw ~= nil then
--						animalSubType.input.straw.numKeyframes = k;
					end;

-- milkOutput

					k = 0;
					while true do
						local xmlKeyMilkOutput = string.format(xmlKeySubType..".milkOutputs.milkOutput(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyMilkOutput) then
						if isDebug then print(xmlKeyMilkOutput.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyMilkOutput .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyMilkOutput .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyMilkOutput .. "#litersPerMonth");
						local litersPerMonth = getXMLFloat(xmlFile, xmlKeyMilkOutput .. "#litersPerMonth");
--						if isDebug then print(litersPerMonth); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyMilkOutput .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.output.milk[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.output.milk[animalSubTypeNameXML][1][k] 		= litersPerMonth;
						RN.animalSummerScaling.output.milk[animalSubTypeNameXML][k] 		= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." litersPerMonth = "..litersPerMonth.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.output.milk ~= nil then
--						animalSubType.output.milk.numKeyframes = k;
					end;

-- liquidManureOutput

					k = 0;
					while true do
						local xmlKeyLiquidManureOutput = string.format(xmlKeySubType..".liquidManureOutputs.liquidManureOutput(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyLiquidManureOutput) then
						if isDebug then print(xmlKeyLiquidManureOutput.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyLiquidManureOutput .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyLiquidManureOutput .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyLiquidManureOutput .. "#litersPerMonth");
						local litersPerMonth = getXMLFloat(xmlFile, xmlKeyLiquidManureOutput .. "#litersPerMonth");
--						if isDebug then print(litersPerMonth); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyLiquidManureOutput .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.output.liquidManure[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.output.liquidManure[animalSubTypeNameXML][1][k] 		= litersPerMonth;
						RN.animalSummerScaling.output.liquidManure[animalSubTypeNameXML][k] 	= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." litersPerMonth = "..litersPerMonth.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.output.liquidManure ~= nil then
--						animalSubType.output.liquidManure.numKeyframes = k;
					end;

-- manureOutput

					k = 0;
					while true do
						local xmlKeyManureOutput = string.format(xmlKeySubType..".manureOutputs.manureOutput(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyManureOutput) then
						if isDebug then print(xmlKeyManureOutput.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyManureOutput .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyManureOutput .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyManureOutput .. "#litersPerMonth");
						local litersPerMonth = getXMLFloat(xmlFile, xmlKeyManureOutput .. "#litersPerMonth");
--						if isDebug then print(litersPerMonth); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyManureOutput .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.output.manure[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.output.manure[animalSubTypeNameXML][1][k] 		= litersPerMonth;
						RN.animalSummerScaling.output.manure[animalSubTypeNameXML][k] 		= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." litersPerMonth = "..litersPerMonth.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.output.manure ~= nil then
--						animalSubType.output.manure.numKeyframes = k;
					end;
-- palletOutput

					k = 0;
					while true do
						local xmlKeyPalletOutput = string.format(xmlKeySubType..".palletOutputs.palletOutput(%d)", k);
						if not hasXMLProperty(xmlFile, xmlKeyPalletOutput) then
						if isDebug then print(xmlKeyPalletOutput.." end of list or does not have xml property"); end;
							break;
						end;
--						print(xmlKeyPalletOutput .. "#age");
						local age = getXMLInt(xmlFile, xmlKeyPalletOutput .. "#age");
--						if isDebug then print(age); end;
--						print(xmlKeyPalletOutput .. "#litersPerMonth");
						local litersPerMonth = getXMLFloat(xmlFile, xmlKeyPalletOutput .. "#litersPerMonth");
--						if isDebug then print(litersPerMonth); end;
						local grazingPeriodScaleFactor = getXMLFloat(xmlFile, xmlKeyPalletOutput .. "#grazingPeriodScaleFactor");
--						if isDebug then print(grazingPeriodScaleFactor); end;
						k = k + 1;
						RN.animalScaling.output.pallets[animalSubTypeNameXML]["time"][k] 	= age;
						RN.animalScaling.output.pallets[animalSubTypeNameXML][1][k] 		= litersPerMonth;
						RN.animalSummerScaling.output.pallets[animalSubTypeNameXML][k] 	= grazingPeriodScaleFactor;
						if isDebug then print(animalSubTypeNameXML.." age = "..age.." litersPerMonth = "..litersPerMonth.. " grazingPeriodScaleFactor = "..grazingPeriodScaleFactor);end;
					end;
					if animalSubType.output.pallets ~= nil then
--						animalSubType.output.pallets.numKeyframes = k;
					end;

					j = j + 1;
				end;
				i = i + 1;
			end;
		else
			print("xmlFile not loaded");
		end;
	end;

	RN.animalValues														= {};	-- tables of set values

	for i, animalSubType in pairs(g_currentMission.animalSystem.subTypes) do
--		print(i);
		if printToLog then print(" "); end;
		if printToLog then print(animalSubType.name); end;
		local animalName = tostring(animalSubType.name);				-- F. ex. Holstein
		if printToLog then print("typeIndex : "..animalSubType.typeIndex); end;


		RN.animalValues[animalName] = {};
		RN.animalValues[animalName].animalName	= animalName;
		RN.animalValues[animalName].typeIndex = animalSubType.typeIndex;
		RN.animalValues[animalName].animalType = tostring(g_currentMission.animalSystem.types[animalSubType.typeIndex].name);
		if printToLog then print("Type : "..RN.animalValues[animalName].animalType); end;

		if RN.animalValues[animalName].animalType == "PIG" then
			animalSubType.reproductionDurationMonth = RN.animalScaling.pigReproductionMonth;
			if printToLog then print("ReproductionDurationMonth = "..animalSubType.reproductionDurationMonth); end;

		end;

-- buyPrice

		if animalSubType.buyPrice ~= nil then
			if RN.animalScaling.buyPrice[animalName] ~= nil then
				if printToLog then print("\n ... buyPrice scaling ... "..currency); end;
				if printToLog then print("keyframes : "..animalSubType.buyPrice.numKeyframes.."\n"); end;
				for j = 1,animalSubType.buyPrice.numKeyframes do
					if printToLog then print("before: Age mth "..animalSubType.buyPrice.keyframes[j]["time"].." : "..animalSubType.buyPrice.keyframes[j][1]); end;
					animalSubType.buyPrice.keyframes[j][1] = RN.animalScaling.buyPrice[animalName][1][j];
					animalSubType.buyPrice.keyframes[j]["time"]	= RN.animalScaling.buyPrice[animalName]["time"][j];
					if printToLog then print("after : Age mth "..animalSubType.buyPrice.keyframes[j]["time"].." : "..animalSubType.buyPrice.keyframes[j][1]); end;
				end;
				animalSubType.buyPrice.maxTime = animalSubType.buyPrice.keyframes[animalSubType.buyPrice.numKeyframes]["time"];
			else
				if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
			end;
		end;

-- sellPrice

		if animalSubType.sellPrice ~= nil then
			if RN.animalScaling.sellPrice[animalName] ~= nil then
				if printToLog then print("\n ... sellPrice scaling ... "..currency); end;
				if printToLog then print("keyframes : "..animalSubType.sellPrice.numKeyframes.."\n"); end;
				for j = 1,animalSubType.sellPrice.numKeyframes do
					if printToLog then print("before: Age mth "..animalSubType.sellPrice.keyframes[j]["time"].." : "..animalSubType.sellPrice.keyframes[j][1]); end;
					animalSubType.sellPrice.keyframes[j][1] = RN.animalScaling.sellPrice[animalName][1][j];
					animalSubType.sellPrice.keyframes[j]["time"]	= RN.animalScaling.sellPrice[animalName]["time"][j];
					if printToLog then print("after : Age mth "..animalSubType.sellPrice.keyframes[j]["time"].." : "..animalSubType.sellPrice.keyframes[j][1]); end;
				end;
				animalSubType.sellPrice.maxTime = animalSubType.sellPrice.keyframes[animalSubType.sellPrice.numKeyframes]["time"];
			else
				if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
			end;
		end;

-- transportPrice

		if animalSubType.transportPrice ~= nil then
			if RN.animalScaling.transportPrice[animalName] ~= nil then
				if printToLog then print("\n ... transportPrice scaling ... "..currency); end;
				if printToLog then print("keyframes : "..animalSubType.transportPrice.numKeyframes.."\n"); end;
				for j = 1,animalSubType.transportPrice.numKeyframes do
					if printToLog then print("before: Age mth "..animalSubType.transportPrice.keyframes[j]["time"].." : "..animalSubType.transportPrice.keyframes[j][1]); end;
					animalSubType.transportPrice.keyframes[j][1] = RN.animalScaling.transportPrice[animalName][1][j];
					animalSubType.transportPrice.keyframes[j]["time"]	= RN.animalScaling.transportPrice[animalName]["time"][j];
					if printToLog then print("after : Age mth "..animalSubType.transportPrice.keyframes[j]["time"].." : "..animalSubType.transportPrice.keyframes[j][1]); end;
				end;
				animalSubType.transportPrice.maxTime = animalSubType.transportPrice.keyframes[animalSubType.transportPrice.numKeyframes]["time"];
			else
				if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
			end;
		end;

		if animalSubType.input ~= nil then

			RN.animalValues[animalName].input = {};

-- water input

			if animalSubType.input.water ~= nil then
				if RN.animalScaling.input.water[animalName] ~= nil then
					if printToLog then print("\n ... input/water scaling (Liters/month) ..."); end;
					if printToLog then print("keyframes : "..animalSubType.input.water.numKeyframes.."\n"); end;
					for j = 1,animalSubType.input.water.numKeyframes do
						if printToLog then print("before: Age mth "..animalSubType.input.water.keyframes[j]["time"].." : "..animalSubType.input.water.keyframes[j][1]); end;
						animalSubType.input.water.keyframes[j][1] = RN.animalScaling.input.water[animalName][1][j];
						animalSubType.input.water.keyframes[j]["time"]	= RN.animalScaling.input.water[animalName]["time"][j];
						if printToLog then print("after : Age mth "..animalSubType.input.water.keyframes[j]["time"].." : "..animalSubType.input.water.keyframes[j][1]); end;
					end;
					animalSubType.input.water.maxTime = animalSubType.input.water.keyframes[animalSubType.input.water.numKeyframes]["time"];
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no water input - "); end;
			end;

-- food input

			if animalSubType.input.food ~= nil then
				if RN.animalScaling.input.food[animalName] ~= nil then
					if printToLog then print("\n ... input/food scaling (Liters/month) ..."); end;
					if printToLog then print("keyframes : "..animalSubType.input.food.numKeyframes.."\n"); end;
					for j = 1,animalSubType.input.food.numKeyframes do
						if printToLog then print("before: Age mth "..animalSubType.input.food.keyframes[j]["time"].." : "..animalSubType.input.food.keyframes[j][1]); end;
						animalSubType.input.food.keyframes[j][1] = RN.animalScaling.input.food[animalName][1][j];
						animalSubType.input.food.keyframes[j]["time"]	= RN.animalScaling.input.food[animalName]["time"][j];
						if printToLog then print("after : Age mth "..animalSubType.input.food.keyframes[j]["time"].." : "..animalSubType.input.food.keyframes[j][1]); end;
					end;
					animalSubType.input.food.maxTime = animalSubType.input.food.keyframes[animalSubType.input.food.numKeyframes]["time"];
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no food input - "); end;
			end;

-- straw input

			if animalSubType.input.straw ~= nil then
				if RN.animalScaling.input.straw[animalName] ~= nil then
					if printToLog then print("\n ... input/straw scaling (Liters/month) ... \n"); end;
					if printToLog then print("keyframes : "..animalSubType.input.straw.numKeyframes.."\n"); end;
					for j = 1,animalSubType.input.straw.numKeyframes do
						if printToLog then print("before: Age mth "..animalSubType.input.straw.keyframes[j]["time"].." : "..animalSubType.input.straw.keyframes[j][1]); end;
						animalSubType.input.straw.keyframes[j][1] = RN.animalScaling.input.straw[animalName][1][j];
						animalSubType.input.straw.keyframes[j]["time"]	= RN.animalScaling.input.straw[animalName]["time"][j];
						if printToLog then print("after : Age mth "..animalSubType.input.straw.keyframes[j]["time"].." : "..animalSubType.input.straw.keyframes[j][1]); end;
					end;
					animalSubType.input.straw.maxTime = animalSubType.input.straw.keyframes[animalSubType.input.straw.numKeyframes]["time"];
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no straw input - "); end;
			end;


		end;

		if animalSubType.output ~= nil then

			RN.animalValues[animalName].output = {};

-- milk output

			if animalSubType.output.milk ~= nil then
				if RN.animalScaling.output.milk[animalName] ~= nil then
					if printToLog then print("\n ... output/milk scaling (Liters/month) ..."); end;
					if printToLog then print("keyframes : "..animalSubType.output.milk.numKeyframes.."\n"); end;
					for j = 1,animalSubType.output.milk.numKeyframes do
						if animalSubType.output.milk.keyframes[j] ~= nil then
							if printToLog then print("before: Age mth "..animalSubType.output.milk.keyframes[j]["time"].." : "..animalSubType.output.milk.keyframes[j][1]); end;
							animalSubType.output.milk.keyframes[j][1] = RN.animalScaling.output.milk[animalName][1][j];
							animalSubType.output.milk.keyframes[j]["time"]	= RN.animalScaling.output.milk[animalName]["time"][j];
							if printToLog then print("after : Age mth "..animalSubType.output.milk.keyframes[j]["time"].." : "..animalSubType.output.milk.keyframes[j][1]); end;
							animalSubType.output.milk.maxTime = animalSubType.output.milk.keyframes[j]["time"];
						end;
					end;					
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no milk output - "); end;
			end;

-- liquid manure output

			if animalSubType.output.liquidManure ~= nil then
				if RN.animalScaling.output.liquidManure[animalName] ~= nil then
					if printToLog then print("\n ... output/liquidManure scaling (Liters/month) ..."); end;
					if printToLog then print("keyframes : "..animalSubType.output.liquidManure.numKeyframes.."\n"); end;
					for j = 1,animalSubType.output.liquidManure.numKeyframes do
						if printToLog then print("before: Age mth "..animalSubType.output.liquidManure.keyframes[j]["time"].." : "..animalSubType.output.liquidManure.keyframes[j][1]); end;
						animalSubType.output.liquidManure.keyframes[j][1] = RN.animalScaling.output.liquidManure[animalName][1][j];
						animalSubType.output.liquidManure.keyframes[j]["time"]	= RN.animalScaling.output.liquidManure[animalName]["time"][j];
						if printToLog then print("after : Age mth "..animalSubType.output.liquidManure.keyframes[j]["time"].." : "..animalSubType.output.liquidManure.keyframes[j][1]); end;
					end;
					animalSubType.output.liquidManure.maxTime = animalSubType.output.liquidManure.keyframes[animalSubType.output.liquidManure.numKeyframes]["time"];
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no liquidManure output - "); end;
			end;

-- manure output

			if animalSubType.output.manure ~= nil then
				if RN.animalScaling.output.manure[animalName] ~= nil then
					if printToLog then print("\n ... output/manure scaling (Liters/month) ... "); end;
					if printToLog then print("keyframes : "..animalSubType.output.manure.numKeyframes.."\n"); end;
					for j = 1,animalSubType.output.manure.numKeyframes do
						if printToLog then print("before: Age mth "..animalSubType.output.manure.keyframes[j]["time"].." : "..animalSubType.output.manure.keyframes[j][1]); end;
						animalSubType.output.manure.keyframes[j][1] = RN.animalScaling.output.manure[animalName][1][j];
						animalSubType.output.manure.keyframes[j]["time"]	= RN.animalScaling.output.manure[animalName]["time"][j];
						if printToLog then print("after : Age mth "..animalSubType.output.manure.keyframes[j]["time"].." : "..animalSubType.output.manure.keyframes[j][1]); end;
					end;
					animalSubType.output.manure.maxTime = animalSubType.output.manure.keyframes[animalSubType.output.manure.numKeyframes]["time"];
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no manure output - "); end;
			end;

-- pallets output

			if animalSubType.output.pallets ~= nil then
				if RN.animalScaling.output.pallets[animalName] ~= nil then
					if printToLog then print("\n ... output/pallets scaling (Liters/month) ... "); end;
					if printToLog then print("keyframes : "..animalSubType.output.pallets.numKeyframes.."\n"); end;
					for j = 1,animalSubType.output.pallets.numKeyframes do
						if printToLog then print("before: Age mth "..animalSubType.output.pallets.keyframes[j]["time"].." : "..animalSubType.output.pallets.keyframes[j][1]); end;
						animalSubType.output.pallets.keyframes[j][1] = RN.animalScaling.output.pallets[animalName][1][j];
						animalSubType.output.pallets.keyframes[j]["time"]	= RN.animalScaling.output.pallets[animalName]["time"][j];
						if printToLog then print("after : Age mth "..animalSubType.output.pallets.keyframes[j]["time"].." : "..animalSubType.output.pallets.keyframes[j][1]); end;
					end;
					animalSubType.output.pallets.maxTime = animalSubType.output.pallets.keyframes[animalSubType.output.pallets.numKeyframes]["time"];
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no pallets output - "); end;
			end;
		else
				if printToLog then print(" - animal has no output - "); end;
		end;
	end;


	for i, animalSubType in pairs(g_currentMission.animalSystem.subTypes) do
		if printToLog then print(" "); end;
		if printToLog then print(animalSubType.name); end;
		local animalName = tostring(animalSubType.name);
		local subTypeIndex = animalSubType.subTypeIndex;
		local animalTypeName = RN.animalScaling.animalSubTypes[subTypeIndex].subTypeTypeNameMAP
		local FirstGrazingMonth = RN.animalScaling.FirstGrazingMonth[animalTypeName];
		local LastGrazingMonth  = RN.animalScaling.LastGrazingMonth[animalTypeName];
		local isGrazingPeriod = ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth));
		RN.animalScaling.isGrazingPeriod = isGrazingPeriod;
		if printToLog then print("typeIndex : "..animalSubType.typeIndex); end;
		if printToLog then print("Type : "..RN.animalValues[animalName].animalType); end;
		print("\nGrazing months: "..FirstGrazingMonth.." to "..LastGrazingMonth);
		if isGrazingPeriod then print("Grazing season"); end;

-- buyPrice

		if animalSubType.buyPrice ~= nil then
			if RN.animalScaling.buyPrice[animalName] ~= nil then
				if printToLog then print("\n ... buyPrice grazing period scaling ... "..currency.." \n"); end;
				for j = 1,animalSubType.buyPrice.numKeyframes do
					if printToLog then print("Age group "..j.." non-grazing : "..round(10 * RN.animalScaling.buyPrice[animalName][1][j])/10); end;
					if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
						animalSubType.buyPrice.keyframes[j][1] = round(10 * RN.animalSummerScaling.buyPrice[animalName][j] * RN.animalScaling.buyPrice[animalName][1][j])/10;
					else
						animalSubType.buyPrice.keyframes[j][1] = round(10 * RN.animalScaling.buyPrice[animalName][1][j])/10;
					end;
					if printToLog then print("Age group "..j.." grazing     : "..animalSubType.buyPrice.keyframes[j][1]); end;
				end;
			else
				if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
			end;
		end;

-- sellPrice

		if animalSubType.sellPrice ~= nil then
			if RN.animalScaling.sellPrice[animalName] ~= nil then
				if printToLog then print("\n ... sellPrice grazing period scaling ... "..currency.." \n"); end;
				for j = 1,animalSubType.sellPrice.numKeyframes do
					if printToLog then print("Age group "..j.." non-grazing : "..round(10 * RN.animalScaling.sellPrice[animalName][1][j])/10); end;
					if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
						animalSubType.sellPrice.keyframes[j][1] =  round(10 * RN.animalSummerScaling.sellPrice[animalName][j] * RN.animalScaling.sellPrice[animalName][1][j])/10;
					else
						animalSubType.sellPrice.keyframes[j][1] = round(10 * RN.animalScaling.sellPrice[animalName][1][j])/10;
					end;
					if printToLog then print("Age group "..j.." grazing     : "..animalSubType.sellPrice.keyframes[j][1]); end;
				end;
			else
				if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
			end;
		end;

-- transportPrice

		if animalSubType.transportPrice ~= nil then
			if RN.animalScaling.transportPrice[animalName] ~= nil then
				if printToLog then print("\n ... transportPrice grazing period scaling ... "..currency.." \n"); end;
				for j = 1,animalSubType.transportPrice.numKeyframes do
					if printToLog then print("Age group "..j.." non-grazing : "..round(10 * RN.animalScaling.transportPrice[animalName][1][j])/10); end;
					if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
						animalSubType.transportPrice.keyframes[j][1] =  round(10 * RN.animalSummerScaling.transportPrice[animalName][j] * RN.animalScaling.transportPrice[animalName][1][j])/10;
					else
						animalSubType.transportPrice.keyframes[j][1] =  round(10 * RN.animalScaling.transportPrice[animalName][1][j])/10;
					end;
					if printToLog then print("Age group "..j.." grazing     : "..animalSubType.transportPrice.keyframes[j][1]); end;
				end;
			else
				if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
			end;
		end;

		if animalSubType.input ~= nil then

-- water input

			if animalSubType.input.water ~= nil then
				if RN.animalScaling.input.water[animalName] ~= nil then
					if printToLog then print("\n ... input/water grazing period scaling  (Liters/month) ... \n"); end;
					for j = 1,animalSubType.input.water.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..round(10 * RN.animalScaling.input.water[animalName][1][j])/10); end;
						if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
							animalSubType.input.water.keyframes[j][1] = round(10 * RN.animalSummerScaling.input.water[animalName][j] * RN.animalScaling.input.water[animalName][1][j])/10;
						else
							animalSubType.input.water.keyframes[j][1] = round(10 * RN.animalScaling.input.water[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.input.water.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no water input - "); end;
			end;

-- food input

			if animalSubType.input.food ~= nil then
				if RN.animalScaling.input.food[animalName] ~= nil then
					if printToLog then print("\n ... input/food grazing period scaling  (Liters/month) ... \n"); end;
					for j = 1,animalSubType.input.food.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..round(10 * RN.animalScaling.input.food[animalName][1][j])/10); end;
						if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
							animalSubType.input.food.keyframes[j][1] =  round(10 * RN.animalSummerScaling.input.food[animalName][j] * RN.animalScaling.input.food[animalName][1][j])/10;
						else
							animalSubType.input.food.keyframes[j][1] =  round(10 * RN.animalScaling.input.food[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.input.food.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no food input - "); end;
			end;

-- straw input

			if animalSubType.input.straw ~= nil then
				if RN.animalScaling.input.straw[animalName] ~= nil then
					if printToLog then print("\n ... input/straw grazing period scaling  (Liters/month) ... \n"); end;
					for j = 1,animalSubType.input.straw.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..round(10 * RN.animalScaling.input.straw[animalName][1][j])/10); end;
						if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
							animalSubType.input.straw.keyframes[j][1] = round(10 * RN.animalSummerScaling.input.straw[animalName][j] * RN.animalScaling.input.straw[animalName][1][j])/10;
						else
							animalSubType.input.straw.keyframes[j][1] =   round(10 * RN.animalScaling.input.straw[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.input.straw.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no straw input - "); end;
			end;

		end;


		if animalSubType.output ~= nil then

-- milk output

			if animalSubType.output.milk ~= nil then
				if RN.animalScaling.output.milk[animalName] ~= nil then
					if printToLog then print("\n ... output/milk grazing period scaling  (Liters/month) ... \n"); end;
					for j = 1,animalSubType.output.milk.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..round(10 * RN.animalScaling.output.milk[animalName][1][j])/10); end;
						if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
							animalSubType.output.milk.keyframes[j][1] =  round(10 * RN.animalSummerScaling.output.milk[animalName][j] * RN.animalScaling.output.milk[animalName][1][j])/10;
						else
							animalSubType.output.milk.keyframes[j][1] =  round(10 * RN.animalScaling.output.milk[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.milk.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no milk output - "); end;
			end;

-- liquidManure output

			if animalSubType.output.liquidManure ~= nil then
				if RN.animalScaling.output.liquidManure[animalName] ~= nil then
					if printToLog then print("\n ... output/liquidManure grazing period scaling  (Liters/month) ... \n"); end;
					for j = 1,animalSubType.output.liquidManure.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..round(10 * RN.animalScaling.output.liquidManure[animalName][1][j])/10); end;
						if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
							animalSubType.output.liquidManure.keyframes[j][1] =  round(10 * RN.animalSummerScaling.output.liquidManure[animalName][j] * RN.animalScaling.output.liquidManure[animalName][1][j])/10;
						else
							animalSubType.output.liquidManure.keyframes[j][1] =  round(10 * RN.animalScaling.output.liquidManure[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.liquidManure.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no liquidManure output - "); end;
			end;

-- manure output

			if animalSubType.output.manure ~= nil then
				if RN.animalScaling.output.manure[animalName] ~= nil then
					if printToLog then print("\n ... output/manure grazing period scaling  (Liters/month) ... \n"); end;
					for j = 1,animalSubType.output.manure.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..round(10 * RN.animalScaling.output.manure[animalName][1][j])/10); end;
						if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
							animalSubType.output.manure.keyframes[j][1] =  round(10 * RN.animalSummerScaling.output.manure[animalName][j] * RN.animalScaling.output.manure[animalName][1][j])/10;
						else
							animalSubType.output.manure.keyframes[j][1] =  round(10 * RN.animalScaling.output.manure[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.manure.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no manure output - "); end;
			end;	-- manure output

-- pallets output

			if animalSubType.output.pallets ~= nil then
				if RN.animalScaling.output.pallets[animalName] ~= nil then
					if printToLog then print("\n ... output/pallets grazing period scaling  (Liters/month) ... \n"); end;
					for j = 1,animalSubType.output.pallets.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..round(10 * RN.animalScaling.output.pallets[animalName][1][j])/10); end;
						if ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth)) then
							animalSubType.output.pallets.keyframes[j][1] =  round(10 * RN.animalSummerScaling.output.pallets[animalName][j] * RN.animalScaling.output.pallets[animalName][1][j])/10;
						else
							animalSubType.output.pallets.keyframes[j][1] =  round(10 * RN.animalScaling.output.pallets[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.pallets.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			else
				if printToLog then print(" - animal has no pallets output - "); end;
			end;	-- pallets output
		else
			if printToLog then print(" - animal has no output - "); end;
		end;	-- output
	end;	-- for loop animalType
	print("\n --- RealNumbersAnimalScaling.lua finished --- \n");
end;


function RealNumbersAnimalScaling:keyEvent(unicode, sym, modifier, isDown)
	local currentMonth = (g_currentMission.environment.currentPeriod + 2) % 12; 	-- March is apparently month 1
	if currentMonth == 0 then currentMonth = 12; end;
--	local FirstGrazingMonth = RN.animalScaling.FirstGrazingMonth[animalTypeName]
--	local LastGrazingMonth  = RN.animalScaling.LastGrazingMonth;
	local printToLog = true;		-- set to false is no output should be printed to log file.

	if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_r) and Input.isKeyPressed(Input.KEY_a) then

				if printToLog then print("\n --- RealNumbersAnimalScaling lAlt-ra pressed --- \n"); end;
				if printToLog then print("\n Month = "..currentMonth); end;


				for i, animalSubType in pairs(g_currentMission.animalSystem.subTypes) do
					if printToLog then print(" "); end;

					if printToLog then print(animalSubType.name); end;
					local animalName = tostring(animalSubType.name);
--					local animalTypeName = RN.animalScaling.animalName[animalName];
					local animalTypeName = RN.animalValues[animalName].animalType;
					local FirstGrazingMonth = RN.animalScaling.FirstGrazingMonth[animalTypeName];
					local LastGrazingMonth  = RN.animalScaling.LastGrazingMonth[animalTypeName];
					local isGrazingPeriod = ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth));
					RN.animalScaling.isGrazingPeriod = isGrazingPeriod;

					if printToLog then print("typeIndex : "..animalSubType.typeIndex); end;
					if printToLog then print("Type : "..RN.animalValues[animalName].animalType); end;
					print("\nGrazing months: "..FirstGrazingMonth.." to "..LastGrazingMonth);
					if isGrazingPeriod then print("Grazing season"); end;

-- buyPrice

					if animalSubType.buyPrice ~= nil then
						if RN.animalScaling.buyPrice[animalName] ~= nil then
							if printToLog then print("\n ... buyPrice scaling ... \n"); end;
							for j = 1,animalSubType.buyPrice.numKeyframes do
								if printToLog then print("Age group "..j.." Non-grazing period : "..round(10 * RN.animalScaling.buyPrice[animalName][1][j])/10); end;
								if printToLog then print("Age group "..j.." Current period      : "..animalSubType.buyPrice.keyframes[j][1]); end;
							end;
						else
							if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
						end;
					end;

-- sellPrice

					if animalSubType.sellPrice ~= nil then
						if RN.animalScaling.sellPrice[animalName] ~= nil then
							if printToLog then print("\n ... sellPrice scaling ... \n"); end;
							for j = 1,animalSubType.sellPrice.numKeyframes do
								if printToLog then print("Age group "..j.." Non-grazing period: "..round(10 * RN.animalScaling.sellPrice[animalName][1][j])/10); end;
								if printToLog then print("Age group "..j.." Current period     : "..animalSubType.sellPrice.keyframes[j][1]); end;
							end;
						else
							if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
						end;
					end;

-- transportPrice

					if animalSubType.transportPrice ~= nil then
						if RN.animalScaling.transportPrice[animalName] ~= nil then
							if printToLog then print("\n ... transportPrice scaling ... \n"); end;
							for j = 1,animalSubType.transportPrice.numKeyframes do
								if printToLog then print("Age group "..j.." Non-grazing period: "..round(10 * RN.animalScaling.transportPrice[animalName][1][j])/10); end;
								if printToLog then print("Age group "..j.." Current period     : "..animalSubType.transportPrice.keyframes[j][1]); end;
							end;
						else
							if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
						end;
					end;

					if animalSubType.input ~= nil then

-- water input

						if animalSubType.input.water ~= nil then
							if RN.animalScaling.input.water[animalName] ~= nil then
								if printToLog then print("\n ... input/water scaling (Liters/month) ... \n"); end;
								for j = 1,animalSubType.input.water.numKeyframes do
									if printToLog then print("Age group "..j.." Non-grazing period (Liters/month) : "..round(10 * RN.animalScaling.input.water[animalName][1][j])/10); end;
									if printToLog then print("Age group "..j.." Current period      (Liters/month) : "..animalSubType.input.water.keyframes[j][1]); end;
								end;
							else
								if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
							end;
						else
							if printToLog then print(" - animal has no water input - "); end;
						end;

-- food input

						if animalSubType.input.food ~= nil then
							if RN.animalScaling.input.food[animalName] ~= nil then
								if printToLog then print("\n ... input/food scaling (Liters/month) ... \n"); end;
								for j = 1,animalSubType.input.food.numKeyframes do
									if printToLog then print("Age group "..j.." Non-grazing period (Liters/month) : "..round(10 * RN.animalScaling.input.food[animalName][1][j])/10); end;
									if printToLog then print("Age group "..j.." Current period      (Liters/month) : "..animalSubType.input.food.keyframes[j][1]); end;
								end;
							else
								if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
							end;
						else
							if printToLog then print(" - animal has no food input - "); end;
						end;

-- straw input

						if animalSubType.input.straw ~= nil then
							if RN.animalScaling.input.straw[animalName] ~= nil then
								if printToLog then print("\n ... input/straw scaling (Liters/month) ... \n"); end;
								for j = 1,animalSubType.input.straw.numKeyframes do
									if printToLog then print("Age group "..j.." Non-grazing period (Liters/month) : "..round(10 * RN.animalScaling.input.straw[animalName][1][j])/10); end;
									if printToLog then print("Age group "..j.." Current period      (Liters/month) : "..animalSubType.input.straw.keyframes[j][1]); end;
								end;
							else
								if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
							end;
						else
							if printToLog then print(" - animal has no straw input - "); end;
						end;

					end;


					if animalSubType.output ~= nil then

-- milk output

						if animalSubType.output.milk ~= nil then
							if RN.animalScaling.output.milk[animalName] ~= nil then
								if printToLog then print("\n ... output/milk scaling (Liters/month) ... \n"); end;
								for j = 1,animalSubType.output.milk.numKeyframes do
									if printToLog then print("Age group "..j.." Non-grazing period (Liters/month) : "..round(10 * RN.animalScaling.output.milk[animalName][1][j])/10); end;
									if printToLog then print("Age group "..j.." Current period      (Liters/month) : "..animalSubType.output.milk.keyframes[j][1]); end;
								end;
							else
								if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
							end;
						else
							if printToLog then print(" - animal has no milk output - "); end;
						end;

-- liquidManure output

						if animalSubType.output.liquidManure ~= nil then
							if RN.animalScaling.output.liquidManure[animalName] ~= nil then
								if printToLog then print("\n ... output/liquidManure scaling (Liters/month) ... \n"); end;
								for j = 1,animalSubType.output.liquidManure.numKeyframes do
									if printToLog then print("Age group "..j.." Non-grazing period (Liters/month) : "..round(10 * RN.animalScaling.output.liquidManure[animalName][1][j])/10); end;
									if printToLog then print("Age group "..j.." Current period      (Liters/month) : "..animalSubType.output.liquidManure.keyframes[j][1]); end;
								end;
							else
								if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
							end;
						else
							if printToLog then print(" - animal has no liquidManure output - "); end;
						end;

-- manure output

						if animalSubType.output.manure ~= nil then
							if RN.animalScaling.output.manure[animalName] ~= nil then
								if printToLog then print("\n ... output/manure scaling (Liters/month) ... \n"); end;
								for j = 1,animalSubType.output.manure.numKeyframes do
									if printToLog then print("Age group "..j.." Non-grazing period (Liters/month) : "..round(10 * RN.animalScaling.output.manure[animalName][1][j])/10); end;
									if printToLog then print("Age group "..j.." Current period      (Liters/month) : "..animalSubType.output.manure.keyframes[j][1]); end;
								end;
							else
								if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
							end;
						else
							if printToLog then print(" - animal has no manure output - "); end;
						end;	-- manure output

-- pallets output

						if animalSubType.output.pallets ~= nil then
							if RN.animalScaling.output.pallets[animalName] ~= nil then
								if printToLog then print("\n ... output/pallets scaling ... \n"); end;
								for j = 1,animalSubType.output.pallets.numKeyframes do
									if printToLog then print("Age group "..j.." Non-grazing period (Units/month)  : "..round(10 * RN.animalScaling.output.pallets[animalName][1][j])/10); end;
									if printToLog then print("Age group "..j.." Current period      (Units/month)  : "..animalSubType.output.pallets.keyframes[j][1]); end;
								end;
							else
								if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
							end;
						else
							if printToLog then print(" - animal has no pallets output - "); end;
						end;	-- pallets output
					else
						if printToLog then print(" - animal has no output - "); end;
					end;	-- output
				end;	-- for loop animalType


	end;
end;

function RealNumbersAnimalScaling:update(dt)
	local printToLog = false;		-- set to false if no output should be printed to log file.
	local currentMonth = (g_currentMission.environment.currentPeriod + 2) % 12;	 -- March is apparently month 1
	if currentMonth == 0 then currentMonth = 12; end;


	if currentMonth ~= RN.animalScaling.currentMonth then			-- only run every new month
		RN.animalScaling.currentMonth = currentMonth;			-- update to new month
		print(" ");
		print("RealNumbersAnimalScaling");
		print(" ");
		print("Month = "..currentMonth);


		for i, animalSubType in pairs(g_currentMission.animalSystem.subTypes) do
			print(" ");
			print(animalSubType.name);
			local animalName = string.upper(tostring(animalSubType.name));			-- F. ex. HOLSTEIN
--			local animalTypeName = RN.animalScaling.animalName[animalName];
			local animalTypeName = RN.animalValues[animalName].animalType;
			local FirstGrazingMonth = RN.animalScaling.FirstGrazingMonth[animalTypeName];
			local LastGrazingMonth  = RN.animalScaling.LastGrazingMonth[animalTypeName];
			local isGrazingPeriod = ((currentMonth >= FirstGrazingMonth) and (currentMonth <= LastGrazingMonth));
			RN.animalScaling.isGrazingPeriod = isGrazingPeriod;
			if printToLog then print("typeIndex : "..animalSubType.typeIndex); end;
			if printToLog then print("Type : "..RN.animalValues[animalName].animalType); end;
			print(" ");
			print("Grazing months: "..FirstGrazingMonth.." to "..LastGrazingMonth);
			if isGrazingPeriod then print("Grazing season"); end;

-- buyPrice

			if animalSubType.buyPrice ~= nil then
				if RN.animalScaling.buyPrice[animalName] ~= nil then
					if printToLog then print("\n ... buyPrice grazing period scaling ... \n"); end;
					for j = 1,animalSubType.buyPrice.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing : "..RN.animalScaling.buyPrice[animalName][1][j]); end;
						if isGrazingPeriod then
							animalSubType.buyPrice.keyframes[j][1] = round(10 * RN.animalSummerScaling.buyPrice[animalName][j] * RN.animalScaling.buyPrice[animalName][1][j])/10;
						else
							animalSubType.buyPrice.keyframes[j][1] = round(10 * RN.animalScaling.buyPrice[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing     : "..animalSubType.buyPrice.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			end;

-- sellPrice

			if animalSubType.sellPrice ~= nil then
				if RN.animalScaling.sellPrice[animalName] ~= nil then
					if printToLog then print("\n ... sellPrice grazing period scaling ... \n"); end;
					for j = 1,animalSubType.sellPrice.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing : "..RN.animalScaling.sellPrice[animalName][1][j]); end;
						if isGrazingPeriod then
							animalSubType.sellPrice.keyframes[j][1] = round(10 * RN.animalSummerScaling.sellPrice[animalName][j] * RN.animalScaling.sellPrice[animalName][1][j])/10;
						else
							animalSubType.sellPrice.keyframes[j][1] = round(10 * RN.animalScaling.sellPrice[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing     : "..animalSubType.sellPrice.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			end;

-- transportPrice

			if animalSubType.transportPrice ~= nil then
				if RN.animalScaling.transportPrice[animalName] ~= nil then
					if printToLog then print("\n ... transportPrice grazing period scaling ... \n"); end;
					for j = 1,animalSubType.transportPrice.numKeyframes do
						if printToLog then print("Age group "..j.." non-grazing : "..RN.animalScaling.transportPrice[animalName][1][j]); end;
						if isGrazingPeriod then
							animalSubType.transportPrice.keyframes[j][1] = round(10 * RN.animalSummerScaling.transportPrice[animalName][j] * RN.animalScaling.transportPrice[animalName][1][j])/10;
						else
							animalSubType.transportPrice.keyframes[j][1] = round(10 * RN.animalScaling.transportPrice[animalName][1][j])/10;
						end;
						if printToLog then print("Age group "..j.." grazing     : "..animalSubType.transportPrice.keyframes[j][1]); end;
					end;
				else
					if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
				end;
			end;

			if animalSubType.input ~= nil then

-- water input

				if animalSubType.input.water ~= nil then
					if RN.animalScaling.input.water[animalName] ~= nil then
						if printToLog then print("\n ... input/water grazing period scaling  (Liters/month) ... \n"); end;
						for j = 1,animalSubType.input.water.numKeyframes do
							if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..RN.animalScaling.input.water[animalName][1][j]); end;
							if isGrazingPeriod then
								animalSubType.input.water.keyframes[j][1] = round(10 * RN.animalSummerScaling.input.water[animalName][j] * RN.animalScaling.input.water[animalName][1][j])/10;
							else
								animalSubType.input.water.keyframes[j][1] = round(10 * RN.animalScaling.input.water[animalName][1][j])/10;
							end;
							if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.input.water.keyframes[j][1]); end;
						end;
					else
						if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
					end;
				else
					if printToLog then print(" - animal has no water input - "); end;
				end;

-- food input

				if animalSubType.input.food ~= nil then
					if RN.animalScaling.input.food[animalName] ~= nil then
						if printToLog then print("\n ... input/food grazing period scaling  (Liters/month) ... \n"); end;
						for j = 1,animalSubType.input.food.numKeyframes do
							if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..RN.animalScaling.input.food[animalName][1][j]); end;
							if isGrazingPeriod then
								animalSubType.input.food.keyframes[j][1] = round(10 * RN.animalSummerScaling.input.food[animalName][j] * RN.animalScaling.input.food[animalName][1][j])/10;
							else
								animalSubType.input.food.keyframes[j][1] = round(10 * RN.animalScaling.input.food[animalName][1][j])/10;
							end;
							if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.input.food.keyframes[j][1]); end;
						end;
					else
						if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
					end;
				else
					if printToLog then print(" - animal has no food input - "); end;
				end;

-- straw input

				if animalSubType.input.straw ~= nil then
					if RN.animalScaling.input.straw[animalName] ~= nil then
						if printToLog then print("\n ... input/straw grazing period scaling  (Liters/month) ... \n"); end;
						for j = 1,animalSubType.input.straw.numKeyframes do
							if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..RN.animalScaling.input.straw[animalName][1][j]); end;
							if isGrazingPeriod then
								animalSubType.input.straw.keyframes[j][1] = round(10 * RN.animalSummerScaling.input.straw[animalName][j] * RN.animalScaling.input.straw[animalName][1][j])/10;
							else
								animalSubType.input.straw.keyframes[j][1] = round(10 * RN.animalScaling.input.straw[animalName][1][j])/10;
							end;
							if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.input.straw.keyframes[j][1]); end;
						end;
					else
						if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
					end;
				else
					if printToLog then print(" - animal has no straw input - "); end;
				end;

			end;


			if animalSubType.output ~= nil then

-- milk output

				if animalSubType.output.milk ~= nil then
					if RN.animalScaling.output.milk[animalName] ~= nil then
						if printToLog then print("\n ... output/milk grazing period scaling  (Liters/month) ... \n"); end;
						for j = 1,animalSubType.output.milk.numKeyframes do
							if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..RN.animalScaling.output.milk[animalName][1][j]); end;
							if isGrazingPeriod then
								animalSubType.output.milk.keyframes[j][1] = round(10 * RN.animalSummerScaling.output.milk[animalName][j] * RN.animalScaling.output.milk[animalName][1][j])/10;
							else
								animalSubType.output.milk.keyframes[j][1] = round(10 * RN.animalScaling.output.milk[animalName][1][j])/10;
							end;
							if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.milk.keyframes[j][1]); end;
						end;
					else
						if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
					end;
				else
					if printToLog then print(" - animal has no milk output - "); end;
				end;

-- liquidManure output

				if animalSubType.output.liquidManure ~= nil then
					if RN.animalScaling.output.liquidManure[animalName] ~= nil then
						if printToLog then print("\n ... output/liquidManure grazing period scaling  (Liters/month) ... \n"); end;
						for j = 1,animalSubType.output.liquidManure.numKeyframes do
							if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..RN.animalScaling.output.liquidManure[animalName][1][j]); end;
							if isGrazingPeriod then
								animalSubType.output.liquidManure.keyframes[j][1] = round(10 * RN.animalSummerScaling.output.liquidManure[animalName][j] * RN.animalScaling.output.liquidManure[animalName][1][j])/10;
							else
								animalSubType.output.liquidManure.keyframes[j][1] = round(10 * RN.animalScaling.output.liquidManure[animalName][1][j])/10;
							end;
							if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.liquidManure.keyframes[j][1]); end;
						end;
					else
						if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
					end;
				else
					if printToLog then print(" - animal has no liquidManure output - "); end;
				end;

-- manure output

				if animalSubType.output.manure ~= nil then
					if RN.animalScaling.output.manure[animalName] ~= nil then
						if printToLog then print("\n ... output/manure grazing period scaling  (Liters/month) ... \n"); end;
						for j = 1,animalSubType.output.manure.numKeyframes do
							if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..RN.animalScaling.output.manure[animalName][1][j]); end;
							if isGrazingPeriod then
								animalSubType.output.manure.keyframes[j][1] = round(10 * RN.animalSummerScaling.output.manure[animalName][j] * RN.animalScaling.output.manure[animalName][1][j])/10;
							else
								animalSubType.output.manure.keyframes[j][1] = round(10 * RN.animalScaling.output.manure[animalName][1][j])/10;
							end;
							if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.manure.keyframes[j][1]); end;
						end;
					else
						if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
					end;
				else
					if printToLog then print(" - animal has no manure output - "); end;
				end;	-- manure output

-- pallets output

				if animalSubType.output.pallets ~= nil then
					if RN.animalScaling.output.pallets[animalName] ~= nil then
						if printToLog then print("\n ... output/pallets grazing period scaling  (Liters/month) ... \n"); end;
						for j = 1,animalSubType.output.pallets.numKeyframes do
							if printToLog then print("Age group "..j.." non-grazing  (Liters/month) : "..RN.animalScaling.output.pallets[animalName][1][j]); end;
							if isGrazingPeriod then
								animalSubType.output.pallets.keyframes[j][1] = round(10 * RN.animalSummerScaling.output.pallets[animalName][j] * RN.animalScaling.output.pallets[animalName][1][j])/10;
							else
								animalSubType.output.pallets.keyframes[j][1] = round(10 * RN.animalScaling.output.pallets[animalName][1][j])/10;
							end;
							if printToLog then print("Age group "..j.." grazing      (Liters/month) : "..animalSubType.output.pallets.keyframes[j][1]); end;
						end;
					else
						if printToLog then print("Not defined in RealNumberAnimalScaling"); end;
					end;
				else
					if printToLog then print(" - animal has no pallets output - "); end;
				end;	-- pallets output
			else
				if printToLog then print(" - animal has no output - "); end;
			end;	-- output
		end;	-- for loop animalType
	end;	-- ifNotRunYet

end;
function RealNumbersAnimalScaling:draw()
end;
function RealNumbersAnimalScaling:deleteMap()
end;
function RealNumbersAnimalScaling:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersAnimalScaling);







