--
-- Mod: FS22_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersCropGrowth by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.0.1.0
-- Version 1.0.1.1 - no change
-- Version 1.0.1.2 - Made a few bug fixes so it is possible to have two winter crop cycles
-- Version 1.0.1.3 - no change
-- Version 1.0.1.4 - no change
-- Version 1.0.1.5 - no change
-- Version 1.0.1.6 - no change
-- Version 1.0.1.7 - added multifruit
-- Version 1.0.1.8 - no change
-- Version 1.0.1.9 - no change
-- Version 1.0.2.0 - no change
-- Version 1.0.2.1 - no change
-- Version 1.0.2.2 - no change
-- Version 1.0.2.3 - no change

--[[
	WARNING  --  WARNING --  WARNING  --  WARNING  -- WARNING  --  WARNING  -- WARNING  --  WARNING  -- WARNING  --  WARNING

	If you activate this script on an existing savegame, crop growth states may be affected. 
	It seems that the entire growth cycle is saved at the time of seeding, and that later alterations in crop growth transitions
	will have no effect on crops already seeded. However, my understanding may be incorrect.
	Activating the script in the middle of a growth cycle may alter the harvest month and as a consequence, the crop may 
	reach its harvest state either before or after the harvest month. This issue will disappear when you start seeding the crop
	yourself.

	The safest approach is to start a new savegame.
	
	But if you have done a lot of work on an existing savegame and would not like to start all over, you can still activate the script.
	You just need to understand that it may affect the crop cycle of already seeded crops.
	You may phase in new crop calendars gradually after harvesting a crop. You simply comment out the fruit you do not want to change, or move the 
	entire <fruit > ..... </fruit> group to the comment section at the bottom. 
	
	Maps with disabled crops
	
	Some maps have certain crops disabled in the seed/harvest calendar because that crop is incompatible
	with the local climate. In this case you must either delete that crop from my_cropGrowth.xml or better, cut it
	out and paste it in the comment section at the bottom of that xml-file.
	
--]] 


RealNumbersCropGrowth = {};

function myPrint(text, number, textWidth, unit)
	print(string.format("%s =  %8.0f %s", text..string.rep(" ",textWidth-string.len(text)),number,"    "..unit));
end;

function myPrintFloat(text, number, textWidth, unit)
	print(string.format("%s =  %12.4f %s", text..string.rep(" ",textWidth-string.len(text)),number, unit));
end;
function myPrintString(text, text2, textWidth, unit)
	print(string.format("%s =  %s %s", text..string.rep(" ",textWidth-string.len(text)),text2, unit));
end;
function myPrintGrowthStatesHeader(text, number1, number2, number3, number4, number5, number6, number7, number8, number9, number10, textWidth)
	print(string.format("%s ( c1,  c2  )   =  %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d", text..string.rep(" ",textWidth-string.len(text)),
					number1, number2, number3, number4, number5, number6, number7, number8, number9, number10));
end;
function myPrintGrowthStatesHeader2(text, month1, month2, month3, month4, month5, month6, month7, month8, month9, month10, month11, month12, textWidth)
	print(string.format("%s =  %s %s %s %s %s %s %s %s %s %s %s %s", text..string.rep(" ",textWidth-string.len(text)),
					month1, month2, month3, month4, month5, month6, month7, month8, month9, month10, month11, month12));
end;
function myPrintGrowthStates(text, cycle1state, cycle2state, number1, number2, number3, number4, number5, number6, number7, number8, number9, number10, textWidth)
	print(string.format("%s (%3d, %3d  )   =  %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d", text..string.rep(" ",textWidth-string.len(text)),
					cycle1state, cycle2state, number1, number2, number3, number4, number5, number6, number7, number8, number9, number10));
end;
function myPrintGrowthStates2(text, cycle, number1, number2, number3, number4, number5, number6, number7, number8, number9, number10, number11, number12, textWidth)
	print(string.format("%s   cycle- %3d   %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d", text..string.rep(" ",textWidth-string.len(text)),
					cycle, number1, number2, number3, number4, number5, number6, number7, number8, number9, number10, number11, number12));
end;
function RealNumbersCropGrowth:loadMap(name)
	local textWidth 	= 30;
	local monthNumber 	= 0;
	local growthState 	= 0;
	local growthData 	= {};
	local monthData 	= {};
	local maxGrowthState = 0;
	local fruitName 	= " ";
	local isWinterCrop1 = false;
	local isWinterCrop2 = false;
	local extraDormant1 = false;
	local extraDormant2 = false;
	local dormantMonth1 = 0;
	local dormantMonth2 = 0;
	local dormantMonth11 = 0;
	local dormantMonth12 = 0;
	local dormantMonth13 = 0;
	local dormantMonth21 = 0;
	local dormantMonth22 = 0;
	local dormantMonth23 = 0;
	local numDormantMonths1	= 0;
	local numDormantMonths2	= 0;
	local growthMonths1 = 0;
	local growthMonths2 = 0;
	local extraJump1 	= false;
	local jumpTarget1 	= 0;
	local jumpMonth1 	= 0;
	local extraJump2 	= false;
	local jumpTarget2 	= 0;
	local jumpMonth2 	= 0;
	local initialState1 = 0;
	local initialState2 = 0;
	local useMyXMLfile	= false;
	local cycle1States 	= {};
	local cycle2States 	= {};
	local isDebugging	= false;

	local monthToPeriod = {11, 12, 1, 2, 3, 4, 5,  6,  7,  8, 9, 10};
	local periodToMonth = { 3,  4, 5, 6, 7, 8, 9, 10, 11, 12, 1,  2};
	local monthToName = {"January","February","March","April","May","June","July","August","September","October","November","December"};
	local periodToName = {"March","April","May","June","July","August","September","October","November","December","January","February"};

	local growthStateArray = {};
	print("\n --- RealNumbersCropGrowth.lua started : press lAlt-rg --- \n");
	RN.cropGrowth 		= {};
	RN.notRunYetRealNumbersCropGrowth = true;
	RN.upStartRealNumbersCropGrowth = true;
--[[
	fruitName = "WHEAT";
	RN.cropGrowth[fruitName] = {};
	RN.cropGrowth[fruitName].seedMonth = 9;
	RN.cropGrowth[fruitName].harvestMonth = 7;
	RN.cropGrowth[fruitName].firstDormantMonth = 11;
	RN.cropGrowth[fruitName].lastDormantMonth = 1;
--]]
	local modSettingsDir = getUserProfileAppPath() .. "modSettings";
	if modSettingsDir == nil then
		print(" ");
		print("modSettingsDir undefined");
		print(" ");
	end;

	local savegameIndex = 0;
	if g_currentMission.missionInfo.savegameIndex ~= nil then
		savegameIndex = g_currentMission.missionInfo.savegameIndex;
	end;
	local RNmodSettingsDir = modSettingsDir .. "/RealLifeNumbers/savegame" .. savegameIndex;

	local myCropGrowthXML = RNmodSettingsDir.."/my_cropGrowth.xml";
	if fileExists(myCropGrowthXML) then
		print("\n --- my XML file is used --- \n")
		useMyXMLfile = true;
	else
		print("xml file does not exist for this savegame");
		useMyXMLfile = false;
	end;

--	print("Fruit types on the map \n");
--	DebugUtil.printTableRecursively(g_fruitTypeManager.nameToIndex,"_",0,1);
--	print(" ");
	
	if fileExists(myCropGrowthXML) then
		RN.cropGrowth.isUsed = true;
		local xmlFile = loadXMLFile("fruitTypes", myCropGrowthXML);
		print("Loading file:");
		print(myCropGrowthXML);
		print(" ");

		local m = 0;
		while true do
			local xmlKey = string.format("growth.seasonal.fruit(%d)", m);
			local xmlKey2 = string.format("growth.seasonal.fruit(%d).cycle1", m);
			local xmlKey3 = string.format("growth.seasonal.fruit(%d).cycle2", m);
			local xmlKey4 = string.format("growth.seasonal.fruit(%d).options", m);
			if not hasXMLProperty(xmlFile, xmlKey) then
				print(" End of file or does not have xml property");
				break;
			end;
			fruitName = string.upper(getXMLString(xmlFile, xmlKey .. "#name"));
--			print(fruitName);
			if g_fruitTypeManager.nameToFruitType[fruitName] ~= nil then  -- if fruit is in map
				if fruitName == "UNKNOWN" then    -- To modder: change to fruitName causing issue while debugging
					isDebugging = true; 
				else
					isDebugging = false;
				end;
				RN.cropGrowth[fruitName] = {};
				RN.cropGrowth[fruitName].fruitName = fruitName;
				local seedMonth1 = math.floor(tonumber(getXMLString(xmlFile, xmlKey2 .. "#seedMonth")));
				local seedMonth2 = math.floor(tonumber(getXMLString(xmlFile, xmlKey3 .. "#seedMonth")));
				if isDebugging then print("seedMonth1 = "..seedMonth1); end;
				if isDebugging then print("seedMonth2 = "..seedMonth2); end;
				local harvestMonth1 = math.floor(tonumber(getXMLString(xmlFile, xmlKey2 .. "#harvestMonth")));
				local harvestMonth2 = math.floor(tonumber(getXMLString(xmlFile, xmlKey3 .. "#harvestMonth")));
				if isDebugging then print("harvestMonth1 = "..harvestMonth1); end;
				if isDebugging then print("harvestMonth2 = "..harvestMonth2); end;
				if seedMonth1 < harvestMonth1 then
					isWinterCrop1 = false;
					if isDebugging then print("Cycle 1 is spring crop"); end;
				else
					isWinterCrop1 = true;
					if isDebugging then print("Cycle 1 is winter crop"); end;
				end;
				if seedMonth2 < harvestMonth2 then
					isWinterCrop2 = false;
					if isDebugging then print("Cycle 2 is spring crop"); end;
				else
					isWinterCrop2 = true;
					if isDebugging then print("Cycle 2 is winter crop"); end;
				end;

				dormantMonth11 = 0;
				dormantMonth12 = 0;
				dormantMonth13 = 0;
				dormantMonth21 = 0;
				dormantMonth22 = 0;
				dormantMonth23 = 0;


				local firstDormantMonth1 = math.floor(tonumber(getXMLString(xmlFile, xmlKey2 .. "#firstDormantMonth")));
				local firstDormantMonth2 = math.floor(tonumber(getXMLString(xmlFile, xmlKey3 .. "#firstDormantMonth")));
	--			print(firstDormantMonth1);
				if isDebugging then print("firstDormantMonth1 = "..firstDormantMonth1); end;
				if isDebugging then print("firstDormantMonth2 = "..firstDormantMonth2); end;
				local lastDormantMonth1   = math.floor(tonumber(getXMLString(xmlFile, xmlKey2 .. "#lastDormantMonth")));
				local lastDormantMonth2   = math.floor(tonumber(getXMLString(xmlFile, xmlKey3 .. "#lastDormantMonth")));
	--			print(lastDormantMonth1);
				if isDebugging then print("lastDormantMonth1 = "..lastDormantMonth1); end;
				if isDebugging then print("lastDormantMonth2 = "..lastDormantMonth2); end;
				
				RN.cropGrowth[fruitName].autoGenerate 		= getXMLBool(xmlFile, xmlKey4 .. "#autoGenerate");
				RN.cropGrowth[fruitName].needsRolling 		= getXMLBool(xmlFile, xmlKey4 .. "#needsRolling");
				RN.cropGrowth[fruitName].growthRequiresLime = getXMLBool(xmlFile, xmlKey4 .. "#growthRequiresLime");
				RN.cropGrowth[fruitName].consumesLime 		= getXMLBool(xmlFile, xmlKey4 .. "#consumesLime");

				if isWinterCrop1 then
					growthMonths1 = firstDormantMonth1 - seedMonth1 + harvestMonth1 - lastDormantMonth1;
				else
					growthMonths1 = harvestMonth1 - seedMonth1 + 1;
				end;
				if isWinterCrop2 then
					growthMonths2 = firstDormantMonth2 - seedMonth2 + harvestMonth2 - lastDormantMonth2;
				else
					growthMonths2 = harvestMonth2 - seedMonth2 + 1;
				end;

				local numGrowthStates = g_fruitTypeManager.nameToFruitType[fruitName].numGrowthStates;

				if numGrowthStates < growthMonths1 then
					numDormantMonths1 = growthMonths1 - numGrowthStates;
					extraDormant1 = true;
					dormantMonth1 = seedMonth1 + 1;
					if isWinterCrop1 then
						if numDormantMonths1 == 1 then 
							firstDormantMonth1 = firstDormantMonth1 - 1;		-- Start winter dormancy 1 month earlier 
						end;
						if numDormantMonths1 == 2 then
							firstDormantMonth1 = firstDormantMonth1 - 1;		-- Start winter dormancy 1 month earlier
							lastDormantMonth1 = lastDormantMonth1 + 1;			-- Finish winter dormancy 1 month later				
						end;
						if numDormantMonths1 == 3 then
							firstDormantMonth1 = firstDormantMonth1 - 1;		-- Start winter dormancy 1 month earlier
							lastDormantMonth1 = lastDormantMonth1 + 2;			-- Finish winter dormancy 2 month later				
						end;
					end;
					if not isWinterCrop1 then
						if numDormantMonths1 == 1 then 
							dormantMonth11 = seedMonth1 + 1;
						end;				
						if numDormantMonths1 == 2 then 
							dormantMonth11 = seedMonth1 + 1;
							dormantMonth12 = seedMonth1 + 2;
						end;				
						if numDormantMonths1 == 3 then 
							dormantMonth11 = seedMonth1 + 1;
							dormantMonth12 = seedMonth1 + 2;
							dormantMonth13 = seedMonth1 + 3;
						end;									
					end;
				else
					extraDormant1 = false;
					dormantMonth1 = 0;
				end;
				if numGrowthStates < growthMonths2 then
					numDormantMonths2 = growthMonths2 - numGrowthStates;
					extraDormant2 = true;
					dormantMonth2 = seedMonth2 + 1;
					if isWinterCrop2 then
						if numDormantMonths2 == 1 then 
							firstDormantMonth2 = firstDormantMonth2 - 1;		-- Start winter dormancy 1 month earlier 
						end;
						if numDormantMonths2 == 2 then
							firstDormantMonth2 = firstDormantMonth2 - 1;		-- Start winter dormancy 1 month earlier
							lastDormantMonth2 = lastDormantMonth2 + 1;			-- Finish winter dormancy 1 month later				
						end;
						if numDormantMonths2 == 3 then
							firstDormantMonth2 = firstDormantMonth2 - 1;		-- Start winter dormancy 1 month earlier
							lastDormantMonth2 = lastDormantMonth2 + 2;			-- Finish winter dormancy 2 month later				
						end;
					end;
					if not isWinterCrop2 then
						if numDormantMonths2 == 1 then 
							dormantMonth21 = seedMonth2 + 1;
						end;				
						if numDormantMonths2 == 2 then 
							dormantMonth21 = seedMonth2 + 1;
							dormantMonth22 = seedMonth2 + 2;
						end;				
						if numDormantMonths1 == 3 then 
							dormantMonth21 = seedMonth2 + 1;
							dormantMonth22 = seedMonth2 + 2;
							dormantMonth23 = seedMonth2 + 3;
						end;									
					end;
				else
					extraDormant2 = false;
					dormantMonth2 = 0;
				end;
				if numGrowthStates > growthMonths1 then
					extraJump1 = true;
					jumpTarget1 = numGrowthStates - 1;
					jumpMonth1 = harvestMonth1 - 2;
				else
					extraJump1 = false;
					jumpTarget1 = 0;
					jumpMonth1 = 0;
				end;
				if numGrowthStates > growthMonths2 then
					extraJump2 = true;
					jumpTarget2 = numGrowthStates - 1;
					jumpMonth2 = harvestMonth2 - 2;
				else
					extraJump2 = false;
					jumpTarget2 = 0;
					jumpMonth2 = 0;
				end;

				local fruitIndex = g_fruitTypeManager.nameToIndex[fruitName];
				
				if g_currentMission.growthSystem.seasonalFruitData[fruitIndex] ~= nil then
					growthData = g_currentMission.growthSystem.seasonalFruitData[fruitIndex];
				else
					print(fruitName);
					DebugUtil.printTableRecursively(g_currentMission.growthSystem.seasonalFruitData ,"_",0,3);
					DebugUtil.printTableRecursively(g_currentMission.growthSystem.nonSeasonalFruitData ,"_",0,3);
					do return end;
				end;

				if RN.cropGrowth[fruitName].needsRolling then
					g_fruitTypeManager.nameToFruitType[fruitName].needsRolling = true;
				else
					g_fruitTypeManager.nameToFruitType[fruitName].needsRolling = false;
				end;

				if RN.cropGrowth[fruitName].growthRequiresLime then
					g_fruitTypeManager.nameToFruitType[fruitName].growthRequiresLime = true;
				else
					g_fruitTypeManager.nameToFruitType[fruitName].growthRequiresLime = false;
				end;

				if RN.cropGrowth[fruitName].consumesLime then
					g_fruitTypeManager.nameToFruitType[fruitName].consumesLime = true;
				else
					g_fruitTypeManager.nameToFruitType[fruitName].consumesLime = false;
				end;


				print(" ");
				print(fruitName.." Cycle 1 : Seed month 1 = "..monthToName[seedMonth1].."; Harvest month 1 : "..monthToName[harvestMonth1]);
				print(fruitName.." Cycle 2 : Seed month 2 = "..monthToName[seedMonth2].."; Harvest month 2 : "..monthToName[harvestMonth2]);
				print(" ");
				myPrint("Number of growth states", numGrowthStates, textWidth," ");
				if firstDormantMonth1 > 0 then
					myPrintString("First dormant month 1",monthToName[firstDormantMonth1], textWidth," ");
				end;
				if lastDormantMonth1 > 0 then
					myPrintString("Last dormant month 1",monthToName[lastDormantMonth1], textWidth," ");
				end;
				myPrint("Cycle 1 : Growth months : ",growthMonths1, textWidth," ");
				if firstDormantMonth2 > 0 then
	--				myPrint("First dormant month 2",firstDormantMonth2, textWidth," ");
					myPrintString("First dormant month 2",monthToName[firstDormantMonth2], textWidth," ");
				end;
				if lastDormantMonth2 > 0 then
	--				myPrint("Last dormant month 2",lastDormantMonth2, textWidth," ");
					myPrintString("Last dormant month 2",monthToName[lastDormantMonth2], textWidth," ");
				end;
				myPrint("Cycle 2 : Growth months : ",growthMonths2, textWidth," ");
				if extraDormant1 then
					myPrint("Cycle 1 extra dormant month",dormantMonth1, textWidth," ");
				end;
				if extraDormant2 then
					myPrint("Cycle 2 extra dormant month",dormantMonth2, textWidth," ");
				end;
				if extraJump1 then
					myPrint("Cycle 1 jump target state",jumpTarget1, textWidth," ");
					myPrint("Cycle 1 jump month",jumpMonth1, textWidth," ");
				end;
				if extraJump2 then
					myPrint("Cycle 2 jump target state",jumpTarget2, textWidth," ");
					myPrint("Cycle 2 jump month",jumpMonth2, textWidth," ");
				end;

				local minForageGrowthState = g_fruitTypeManager.nameToFruitType[fruitName].minForageGrowthState;
				local minHarvestingGrowthState = g_fruitTypeManager.nameToFruitType[fruitName].minHarvestingGrowthState;
				local maxHarvestingGrowthState = g_fruitTypeManager.nameToFruitType[fruitName].maxHarvestingGrowthState;
				local witheredState = g_fruitTypeManager.nameToFruitType[fruitName].witheredState;
				local cutState = g_fruitTypeManager.nameToFruitType[fruitName].cutState;
				local maxState = math.max(maxHarvestingGrowthState,cutState);
				myPrint("Forage ready state",minForageGrowthState, textWidth," ");
				myPrint("Min. harvest state",minHarvestingGrowthState, textWidth," ");
				myPrint("Max. harvest state",maxHarvestingGrowthState, textWidth," ");
				myPrint("Withered state",witheredState, textWidth," ");
				myPrint("Cut state",cutState, textWidth," ");
				print(" ");

				print("\n Original growth transistions \n")
				myPrintGrowthStatesHeader("Growth state transitions", 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, textWidth);
				for i = 1,12 do
					monthData = growthData[i];
					growthStateArray = {0,0,0,0,0,0,0,0,0,0,0,0};
					for j = 1,#monthData.growthMapping do
						growthStateArray[j] = monthData.growthMapping[j];
					end
					myPrintGrowthStates(periodToName[i], 0, 0, growthStateArray[1], growthStateArray[2], growthStateArray[3],
						growthStateArray[4], growthStateArray[5], growthStateArray[6], growthStateArray[7],
						growthStateArray[8], growthStateArray[9], growthStateArray[10],textWidth);
				end;
				print(" ");

			-- Redefine growthSystem parameters



				if g_currentMission.growthSystem.seasonalFruitData ~= nil then

	-- cycle 1
					cycle1States = {0,0,0,0,0,0,0,0,0,0,0,0};
					monthNumber = seedMonth1;
					periodIndex = monthToPeriod[monthNumber]; -- initialize counter
					cropState = 1;
					RN.cropGrowth[fruitName].cycle1 = {};

					for i = 1, 12 do


	--					print("\nMonth : "..monthNumber.." Period : "..periodIndex.." Crop state : "..cropState);
	--					print("Original data");
						monthData = growthData[periodIndex];

	--					if monthData.plantingAllowed then print("planting allowed : true"); end;
	--					if not monthData.plantingAllowed then print("planting allowed : false"); end;
	--					if monthData.harvestPossible then print("harvesting posible : true"); end;
	--					if not monthData.harvestPossible then print("harvesting possible : false"); end;

						if monthNumber == seedMonth1  then
							monthData.plantingAllowed = true;
						else
							monthData.plantingAllowed = false;
						end;
						if monthNumber == harvestMonth1 then
							monthData.harvestPossible = true;
						else
							monthData.harvestPossible = false;
						end;

	--					DebugUtil.printTableRecursively(monthData.growthMapping,"_",0,2);

	--					print(" ");
	--					print("New data");
						for k = 1, #monthData.growthMapping do
								monthData.growthMapping[k] = k;
								growthStateArray[k] = k;
						end;

						if ( not isWinterCrop1) then
							if (extraJump1 and (monthNumber == jumpMonth1)) then
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("spring crop jump month : "..monthNumber); end;
								monthData.growthMapping[cropState] = jumpTarget1;
								cycle1States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
								if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
								cropState = jumpTarget1;
							elseif (extraDormant1 and (monthNumber == dormantMonth11 or monthNumber == dormantMonth12 or monthNumber == dormantMonth13)) then						
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("spring crop dormant preharvest month : "..monthNumber); end;
								monthData.growthMapping[cropState] = cropState;
								cycle1States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
								if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
								cropState = cropState;
							else
								if cropState <= numGrowthStates then
									if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
									if isDebugging then print("spring crop normal growth month : "..monthNumber); end;
									monthData.growthMapping[cropState] = cropState + 1;
									cycle1States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
									if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
									cropState = cropState + 1;
								else
									if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
									if isDebugging then print("spring crop stagnation month : "..monthNumber); end;
									monthData.growthMapping[cropState] = cropState ;								
									cycle1States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
									if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
								end;
							end;
						else


							if ((monthNumber >= seedMonth1) and (monthNumber < firstDormantMonth1)) then				-- fall growth
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("winter crop normal growth month : "..monthNumber); end;
								monthData.growthMapping[cropState] = cropState + 1;
								cycle1States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
								if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
								cropState = cropState + 1;
							elseif ((monthNumber >= firstDormantMonth1) or (monthNumber <= lastDormantMonth1)) then	-- winter dormant
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("winter crop dormant month : "..monthNumber); end;
								monthData.growthMapping[cropState] = cropState;
								cycle1States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
								if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
								cropState = cropState;												
							elseif ((monthNumber > lastDormantMonth1) and (monthNumber < seedMonth1)) then		-- spring growth
								if (extraJump1 and monthNumber == jumpMonth1) then
									if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
									if isDebugging then print("winter crop jump month : "..monthNumber); end;
									monthData.growthMapping[cropState] = jumpTarget1;
									cycle1States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
									if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
									cropState = jumpTarget1;
								elseif (extraDormant1 and (monthNumber == dormantMonth1)) then
									if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
									if isDebugging then print("winter crop dormant preharvest month : "..monthNumber); end;
									monthData.growthMapping[cropState] = cropState;
									cycle1States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
									if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
									cropState = cropState;
								else
									if cropState <= numGrowthStates then
										if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
										if isDebugging then print("winter crop normal growth month : "..monthNumber); end;
										monthData.growthMapping[cropState] = cropState + 1;
										cycle1States[periodIndex] = cropState;
										RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
										if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
										cropState = cropState + 1;
									else
										if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
										if isDebugging then print("winter crop stagnation month : "..monthNumber); end;
										monthData.growthMapping[cropState] = cropState;
										cycle1States[periodIndex] = cropState;
										RN.cropGrowth[fruitName].cycle1[monthNumber] = cropState;
										if monthNumber == 8 then initialState1 = monthData.growthMapping[cropState]; end;
									end;
								end;
							end;
						end;


	--					if monthData.plantingAllowed then print("planting allowed : true"); end;
	--					if not monthData.plantingAllowed then print("planting allowed : false"); end;
	--					if monthData.harvestPossible then print("harvesting posible : true"); end;
	--					if not monthData.harvestPossible then print("harvesting possible : false"); end;

	--					DebugUtil.printTableRecursively(monthData.growthMapping,"_",0,3);


						periodIndex = periodIndex + 1;
						if periodIndex > 12 then periodIndex = 1; end;
						monthNumber = (periodIndex + 2) % 12;
						if monthNumber == 0 then monthNumber = 12; end;


					end;


	-- cycle 2

					cycle2States = {0,0,0,0,0,0,0,0,0,0,0,0};
					monthNumber = seedMonth2;
					periodIndex = monthToPeriod[monthNumber]; -- initialize counter
					cropState = 1;
					RN.cropGrowth[fruitName].cycle2 = {};
					for i = 1, 12 do

	--					print("\nMonth : "..monthNumber.." Period : "..periodIndex.." Growth state : "..cropState);
	--					print("Original data");
						monthData = growthData[periodIndex];

	--					if monthData.plantingAllowed then print("planting allowed : true"); end;
	--					if not monthData.plantingAllowed then print("planting allowed : false"); end;
	--					if monthData.harvestPossible then print("harvesting posible : true"); end;
	--					if not monthData.harvestPossible then print("harvesting possible : false"); end;

						if monthNumber == seedMonth2  then
							monthData.plantingAllowed = true;
						end;
						if monthNumber == harvestMonth2 then
							monthData.harvestPossible = true;
						end;

	--					DebugUtil.printTableRecursively(monthData.growthMapping,"_",0,2);

	--					print(" ");
	--					print("New data");

						if ( not isWinterCrop2) then
							if (extraJump2 and (monthNumber == jumpMonth2)) then
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("spring crop jump month : "..monthNumber); end;
								monthData.growthMapping[cropState] = jumpTarget2;
								cycle2States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
								if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
								cropState = jumpTarget2;
							elseif (extraDormant2 and (monthNumber == dormantMonth21 or monthNumber == dormantMonth22 or monthNumber == dormantMonth23)) then
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("spring crop dormant preharvest month : "..monthNumber); end;
								monthData.growthMapping[cropState] = cropState;
								cycle2States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
								if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
								cropState = cropState;
							else
								if cropState <= numGrowthStates then
									if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
									if isDebugging then print("spring crop normal growth month : "..monthNumber); end;
									monthData.growthMapping[cropState] = cropState + 1;
									cycle2States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
									if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
									cropState = cropState + 1;
								else
									if isDebugging then print("spring crop stagnation month : "..monthNumber); end;
									monthData.growthMapping[cropState] = cropState;
									cycle2States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
									if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
								end;
							end;
						else


							if ((monthNumber >= seedMonth2) and (monthNumber < firstDormantMonth2)) then				-- fall growth
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("winter crop normal growth month : "..monthNumber); end;
								monthData.growthMapping[cropState] = cropState + 1;
								cycle2States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
								if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
								cropState = cropState + 1;
							elseif ((monthNumber >= firstDormantMonth2) or (monthNumber <= lastDormantMonth2)) then	-- winter dormant
								if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
								if isDebugging then print("winter crop dormant month : "..monthNumber); end;
								monthData.growthMapping[cropState] = cropState;
								cycle2States[periodIndex] = cropState;
								RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
								if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
								cropState = cropState;	
							elseif ((monthNumber > lastDormantMonth2) and (monthNumber < seedMonth2)) then		-- spring growth
								if (extraJump2 and monthNumber == jumpMonth2) then
									if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
									if isDebugging then print("winter crop jump month : "..monthNumber); end;
									monthData.growthMapping[cropState] = jumpTarget2;
									cycle2States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
									if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
									cropState = jumpTarget2;
								elseif (extraDormant2 and (monthNumber == dormantMonth2)) then
									if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
									if isDebugging then print("winter crop dormant preharvest month : "..monthNumber); end;
									monthData.growthMapping[cropState] = cropState;
									cycle2States[periodIndex] = cropState;
									RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
									if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
									cropState = cropState;
								else
									if cropState <= numGrowthStates then
										if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
										if isDebugging then print("winter crop normal growth month : "..monthNumber); end;
										monthData.growthMapping[cropState] = cropState + 1;
										cycle2States[periodIndex] = cropState;
										RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
										if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
										cropState = cropState + 1;
									else
										if isDebugging then print("Period index : "..periodIndex.."; Crop state : "..cropState); end;
										if isDebugging then print("winter crop stagnation month : "..monthNumber); end;
										monthData.growthMapping[cropState] = cropState;
										cycle2States[periodIndex] = cropState;
										RN.cropGrowth[fruitName].cycle2[monthNumber] = cropState;
										if monthNumber == 8 then initialState2 = monthData.growthMapping[cropState]; end;
									end;
								end;
							end;
						end;


	--					if monthData.plantingAllowed then print("planting allowed : true"); end;
	--					if not monthData.plantingAllowed then print("planting allowed : false"); end;
	--					if monthData.harvestPossible then print("harvesting posible : true"); end;
	--					if not monthData.harvestPossible then print("harvesting possible : false"); end;

	--					DebugUtil.printTableRecursively(monthData.growthMapping,"_",0,3);
						periodIndex = periodIndex + 1;
						if periodIndex > 12 then periodIndex = 1; end;
						monthNumber = (periodIndex + 2) % 12;
						if monthNumber == 0 then monthNumber = 12; end;

					end;
				end;

				print("\n New growth transistions \n")
				myPrintGrowthStatesHeader("Growth state transitions", 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,textWidth);
				for i = 1,12 do
					monthData = growthData[i];
					growthStateArray = {0,0,0,0,0,0,0,0,0,0,0,0};
					for j = 1,#monthData.growthMapping do
						growthStateArray[j] = monthData.growthMapping[j];
					end;
					myPrintGrowthStates(periodToName[i], cycle1States[i],  cycle2States[i], growthStateArray[1], growthStateArray[2], growthStateArray[3],
						growthStateArray[4], growthStateArray[5], growthStateArray[6], growthStateArray[7],
						growthStateArray[8], growthStateArray[9], growthStateArray[10],textWidth);
				end;

				if growthData.initialState ~= nil then
					growthData.initialState[1] = initialState1 - 1;
					growthData.initialState[2] = initialState2 - 1;
				end;

			end;
			m = m + 1;
			
		end;  -- while
	else
		RN.cropGrowth.isUsed = false;
	end; -- if file exists

	print("\n --- RealNumbersCropGrowth.lua finished --- \n");

end;


function RealNumbersCropGrowth:keyEvent(unicode, sym, modifier, isDown)

	local textWidth = 20;

	if RN.cropGrowth.isUsed then

		if Input.isKeyPressed(Input.KEY_lalt) and Input.isKeyPressed(Input.KEY_r) and Input.isKeyPressed(Input.KEY_g) then

			print("\n --- RealNumbersCropGrowth.lua  : pressed lAlt-rg --- \n");

			myPrintGrowthStatesHeader2(" ", "            Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec", textWidth)

			for i, v in pairs(g_currentMission.fruitTypeManager.fruitTypes) do
				local fruitName = string.upper(tostring(g_currentMission.fruitTypeManager.fruitTypes[i].name));
				if RN.cropGrowth[fruitName] ~= nil then
					myPrintGrowthStates2(fruitName, 1, RN.cropGrowth[fruitName].cycle1[1], RN.cropGrowth[fruitName].cycle1[2], RN.cropGrowth[fruitName].cycle1[3], 
					RN.cropGrowth[fruitName].cycle1[4], RN.cropGrowth[fruitName].cycle1[5], RN.cropGrowth[fruitName].cycle1[6], RN.cropGrowth[fruitName].cycle1[7], 
					RN.cropGrowth[fruitName].cycle1[8], RN.cropGrowth[fruitName].cycle1[9], RN.cropGrowth[fruitName].cycle1[10], RN.cropGrowth[fruitName].cycle1[11], 
					RN.cropGrowth[fruitName].cycle1[12], textWidth);
					myPrintGrowthStates2(fruitName, 2, RN.cropGrowth[fruitName].cycle2[1], RN.cropGrowth[fruitName].cycle2[2], RN.cropGrowth[fruitName].cycle2[3], 
					RN.cropGrowth[fruitName].cycle2[4], RN.cropGrowth[fruitName].cycle2[5], RN.cropGrowth[fruitName].cycle2[6], RN.cropGrowth[fruitName].cycle2[7], 
					RN.cropGrowth[fruitName].cycle2[8], RN.cropGrowth[fruitName].cycle2[9], RN.cropGrowth[fruitName].cycle2[10], RN.cropGrowth[fruitName].cycle2[11], 
					RN.cropGrowth[fruitName].cycle2[12], textWidth);
				end;
			end;
			print(" ");
		end;
	end;
end;

function RealNumbersCropGrowth:update(dt)

	if RN.cropGrowth.isUsed then
		local minute = g_currentMission.environment.currentMinute;
		local runMinute = minute % 10; -- remainder
		if runMinute ~= 4 then RN.notRunYetRealNumbersCropGrowth = true; end;
		if runMinute == 4 or RN.upStartRealNumbersCropGrowth then
			if RN.notRunYetRealNumbersCropGrowth then
				if g_fieldManager.farmlandIdFieldMapping ~= nil then
					for i, land in pairs(g_fieldManager.farmlandIdFieldMapping) do
						for k, field in pairs(land) do
	--						local farmland = field.farmland;
	--						local farmlandArea = farmland.areaInHa;
	--						local farmlandId = farmland.id;
	--						local isOwned = farmland.isOwned;
							local fieldId = field.fieldId;
	--						local fieldArea = field.fieldArea;
							local fruitTypeIndex = field.fruitType;
							local fruitType = g_fruitTypeManager:getFruitTypeByIndex(fruitTypeIndex);
							if fruitType ~= nil then
								fruitName = string.upper(tostring(fruitType.name));
								if RN.cropGrowth[fruitName] ~= nil then
									if not RN.cropGrowth[fruitName].autoGenerate then
										if g_fieldManager.fields[fieldId] ~= nil then
											g_fieldManager.fields[fieldId].plannedFruit = 4;			-- switch to OAT
	--										print(fruitName..": Planned fruit has been changed to OAT on field "..fieldId);
										end;
									end;
								end;
							else
								fruitName = "Undefined";
							end;
						end;
					end;
				end;
			end;
			RN.notRunYetRealNumbersCropGrowth = false;
			RN.upStartRealNumbersCropGrowth = false;
		end;
	end;
end;
function RealNumbersCropGrowth:draw()
end;
function RealNumbersCropGrowth:deleteMap()
end;
function RealNumbersCropGrowth:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersCropGrowth);







