--
-- Mod: FS22_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersFillTypes by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.0.1.0 
-- Version 1.0.1.1
--		Prepared to handle filltypes in xml files which are not used by any mod.
--      Needed if you do not use the net wrap add-on or twine add-on mods.
-- Version 1.0.1.2 - no change
-- Version 1.0.1.3 - no change
-- Version 1.0.1.4 - no change Added a few fillTypes from MaizePlus
-- Version 1.0.1.5 - no change
-- Version 1.0.1.6 - no change
-- Version 1.0.1.7 - added multifruit
-- Version 1.0.1.8 - no change
-- Version 1.0.1.9 - no change
-- Version 1.0.2.0 - no change
-- Version 1.0.2.1 - no change
-- Version 1.0.2.2 - updated with carrot, beetroot and parsnip
-- Version 1.0.2.3 - no change

RealNumbersFillTypes = {};

local function myPrintScaledValue(text, number1, number2,textWidth, unit)
	print(string.format("%s : default value =  %12.5f : my value = %12.5f %s", text..string.rep(" ",textWidth-string.len(text)),number1, number2,"    "..unit));
end;

function RealNumbersFillTypes:loadMap(name)
	local currency = "€";	-- or  "$" or "£"
	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;
	RN.fillTypes = {};
	RN.fillTypes.priceScale = 0.333;	-- default RLN value = 0.333 when Economy level = HARD; vanilla game = 1
	local textWidth = 40;
	local useMyXMLfile = true;
	local oldValue = 0;
	local myValue = 0;

		print("\n --- RealNumbersFillTypes.lua started --- \n");

	local modSettingsDir = getUserProfileAppPath() .. "modSettings";
	if modSettingsDir == nil then
		print(" ");
		print("modSettingsDir undefined");
		print(" ");
	end;

	local savegameIndex = 0;
	if g_currentMission.missionInfo.savegameIndex ~= nil then
		savegameIndex = g_currentMission.missionInfo.savegameIndex;
	end;
	local RNmodSettingsDir = modSettingsDir .. "/RealLifeNumbers/savegame" .. savegameIndex;

	local myFillTypeXML = RNmodSettingsDir.."/my_fillTypes.xml";
	if fileExists(myFillTypeXML) then
		print("\n --- my XML file is used --- \n")
		useMyXMLfile = true;
	else
		print("xml file does not exist for this savegame");
		useMyXMLfile = false;
--		createFolder(RNmodSettingsDir);
	end;

	if fileExists(myFillTypeXML) then
			
			local xmlFile = loadXMLFile("fillTypes", myFillTypeXML);
			print("Loading file:");
			print(myFillTypeXML);
			print(" ");

			local i = 0;
			while true do
				local xmlKey = string.format("map.fillTypes.fillType(%d)", i);
				if not hasXMLProperty(xmlFile, xmlKey) then
				print("xmlKey: End of file or does not have xml property");
					break;
				end;
					local fillTypeName = string.upper(getXMLString(xmlFile, xmlKey .. "#name"));
					print(fillTypeName);
					
					local xmlKey2 = string.format("map.fillTypes.fillType(%d).physics", i);
					if hasXMLProperty(xmlFile, xmlKey2) then
						if g_fillTypeManager.nameToFillType[fillTypeName] ~= nil then
							oldValue = g_fillTypeManager.nameToFillType[fillTypeName].massPerLiter;
							myValue = getXMLFloat(xmlFile, xmlKey2.. "#massPerLiter");
							if oldValue ~= nil and myValue ~= nil then
								g_fillTypeManager.nameToFillType[fillTypeName].massPerLiter = myValue/1000;
								myPrintScaledValue(fillTypeName.." massPerLiter",1000*oldValue,myValue,textWidth,"kg/L");
							else
								if oldValue == nil then print(fillTypeName.." : old massPerLiter not defined");end;
								if myValue  == nil then print(fillTypeName.." : my massPerLiter not defined");end;
							end;
						else
							print(fillTypeName.." : not defined");
						end;						
					else
						print("xmlKey 2: End of file or  does not have xml property");
					end;

					local xmlKey3 = string.format("map.fillTypes.fillType(%d).economy", i);
					if hasXMLProperty(xmlFile, xmlKey3) then
						if g_fillTypeManager.nameToFillType[fillTypeName] ~= nil then
							oldValue = g_fillTypeManager.nameToFillType[fillTypeName].pricePerLiter;
							myValue = getXMLFloat(xmlFile, xmlKey3.. "#pricePerLiter");
							if oldValue ~= nil and myValue ~= nil then
								g_fillTypeManager.nameToFillType[fillTypeName].pricePerLiter = myValue;
								myPrintScaledValue(fillTypeName.." pricePerLiter",oldValue,myValue,textWidth,currency.."/L");
							else
								if oldValue == nil then print(fillTypeName.." : old pricePerLiter not defined");end;
								if myValue  == nil then print(fillTypeName.." : my pricePerLiter not defined");end;
							end;
						else
							print(fillTypeName.." : not defined");
						end;
					else
						print("xmlKey 3: End of file or does not have xml property");
					end;
				print(" ");
				i = i + 1;
			end;

	else		

		print("\n --- Simple one-factor parameter scaling in effect --- \n");
		print("\n --- All filltype prices is scaled by "..RN.fillTypes.priceScale.." ---\n");
		
		for i, v in pairs(g_currentMission.fillTypeManager.fillTypes) do
			local fillTypeName = tostring(g_currentMission.fillTypeManager.fillTypes[i].name);
			local oldValue = 0;
			local myValue = 0;
--			print(fillTypeName);
	  
			if g_currentMission.fillTypeManager.fillTypes[i].pricePerLiter  ~= nil then
				oldValue = g_currentMission.fillTypeManager.fillTypes[i].pricePerLiter;
				myValue = oldValue * RN.fillTypes.priceScale;							
				if oldValue ~= nil and myValue ~= nil then
					g_currentMission.fillTypeManager.fillTypes[i].pricePerLiter   = myValue;
					myPrintScaledValue(fillTypeName.." pricePerLiter",oldValue,myValue,textWidth,currency.."/L");
				else
					if oldValue == nil then print(fillTypeName.." : old pricePerLiter not defined");end;
					if myValue  == nil then print(fillTypeName.." : my pricePerLiter not defined");end;
				end;
			end;
			if g_currentMission.fillTypeManager.fillTypes[i].startPricePerLiter   ~= nil then
				oldValue = g_currentMission.fillTypeManager.fillTypes[i].startPricePerLiter;
				myValue = oldValue * RN.fillTypes.priceScale;							
				if oldValue ~= nil and myValue ~= nil then
					g_currentMission.fillTypeManager.fillTypes[i].startPricePerLiter   = myValue;
				else
					if oldValue == nil then print(fillTypeName.." : old startPricePerLiter not defined");end;
					if myValue  == nil then print(fillTypeName.." : my startPricePerLiter not defined");end;
				end;
			end;
			if g_currentMission.fillTypeManager.fillTypes[i].previousHourPrice    ~= nil then
				oldValue = g_currentMission.fillTypeManager.fillTypes[i].previousHourPrice;
				myValue = oldValue * RN.fillTypes.priceScale;							
				if oldValue ~= nil and myValue ~= nil then
					g_currentMission.fillTypeManager.fillTypes[i].previousHourPrice   = myValue;
				else
					if oldValue == nil then print(fillTypeName.." : old previousHourPrice not defined");end;
					if myValue  == nil then print(fillTypeName.." : my previousHourPrice not defined");end;
				end;
			end;					
		end;	
		print("\n --- RealNumbersFillTypes.lua finished --- \n");
	end;
end;


function RealNumbersFillTypes:keyEvent(unicode, sym, modifier, isDown)
end;
function RealNumbersFillTypes:update(dt)
end;
function RealNumbersFillTypes:draw()
end;
function RealNumbersFillTypes:deleteMap()
end;
function RealNumbersFillTypes:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersFillTypes);







