--
-- Mod: FS22_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersFruitTypes by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.0.1.0
-- Version 1.0.1.1 - No change
-- Version 1.0.1.2 - No change
-- Version 1.0.1.3 - no change
-- Version 1.0.1.4 - no change
-- Version 1.0.1.5 - no change
-- Version 1.0.1.6 - no change
-- Version 1.0.1.7 - added multifruit
-- Version 1.0.1.8 - no change
-- Version 1.0.1.9 - no change
-- Version 1.0.2.0 - no change
-- Version 1.0.2.1 - no change
-- Version 1.0.2.2 - updated with beetroot, parsnips and carrot
-- Version 1.0.2.3 - no change

RealNumbersFruitTypes = {};

local function myPrintScaledValue(text, number1, number2,textWidth, unit)
	print(string.format("%s : default value =  %12.5f : my value = %12.5f %s", text..string.rep(" ",textWidth-string.len(text)),number1, number2,"    "..unit));
end;

function RealNumbersFruitTypes:loadMap(name)
	local currency = "€";	-- or  "$" or "£"
	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;
	RN.fruitTypes = {};
	RN.fruitTypes.yieldScale 					= 1;	-- default RLN value = 1
	RN.fruitTypes.yieldScaleWindrow 			= 1;	-- default RLN value = 1
	RN.fruitTypes.seedUsageScale	 			= 0.5;	-- default RLN value = 1
	local textWidth = 40;
	local useMyXMLfile = true;


	 print("\n --- RealNumbersFruitTypes.lua started --- \n");
	


	local modSettingsDir = getUserProfileAppPath() .. "modSettings";
	if modSettingsDir == nil then
		print(" ");
		print("modSettingsDir undefined");
		print(" ");
	end;

	local savegameIndex = 0;
	if g_currentMission.missionInfo.savegameIndex ~= nil then
		savegameIndex = g_currentMission.missionInfo.savegameIndex;
	end;
	local RNmodSettingsDir = modSettingsDir .. "/RealLifeNumbers/savegame" .. savegameIndex;

	local myFruitTypeXML = RNmodSettingsDir.."/my_fruitTypes.xml";
	if fileExists(myFruitTypeXML) then
		print("\n --- my XML file is used --- \n")
		useMyXMLfile = true;
	else
		print("xml file does not exist for this savegame");
		useMyXMLfile = false;
--		createFolder(RNmodSettingsDir);
	end;


	if fileExists(myFruitTypeXML) then
		
		local xmlFile = loadXMLFile("fruitTypes", myFruitTypeXML);
		print("Loading file:");
		print(myFruitTypeXML);
		print(" ");

		local i = 0;
		while true do
			local xmlKey = string.format("map.fruitTypes.fruitType(%d)", i);
			if not hasXMLProperty(xmlFile, xmlKey) then
				print(" does not have xml property");
				break;
			end;
			local fruitName = string.upper(getXMLString(xmlFile, xmlKey .. "#name"));
--			print(fruitName);
			if g_fruitTypeManager.nameToFruitType[fruitName] ~= nil then     -- true if fruit is in map
				local xmlKey2 = string.format("map.fruitTypes.fruitType(%d).cultivation", i);
				if hasXMLProperty(xmlFile, xmlKey2) then
					local oldValue = g_fruitTypeManager.nameToFruitType[fruitName].seedUsagePerSqm;
					local myValue = getXMLFloat(xmlFile, xmlKey2.. "#seedUsagePerSqm");
					if oldValue ~= nil and myValue ~= nil then
						g_fruitTypeManager.nameToFruitType[fruitName].seedUsagePerSqm = myValue;
						myPrintScaledValue(fruitName.." seedUsagePerSqm",oldValue,myValue,textWidth,"L/sqm");
					else
						if oldValue == nil then print(fruitName.." : old seedUsagePerSqm not defined");end;
						if myValue  == nil then print(fruitName.." : my seedUsagePerSqm not defined");end;
					end;
				else
					print("xmlKey 2 does not have xml property");
				end;

				local xmlKey3 = string.format("map.fruitTypes.fruitType(%d).harvest", i);
				if hasXMLProperty(xmlFile, xmlKey3) then
					local oldValue = g_fruitTypeManager.nameToFruitType[fruitName].literPerSqm;
					local myValue = getXMLFloat(xmlFile, xmlKey3.. "#literPerSqm");
					if oldValue ~= nil and myValue ~= nil then
						g_fruitTypeManager.nameToFruitType[fruitName].literPerSqm = myValue;
						myPrintScaledValue(fruitName.." literPerSqm",oldValue,myValue,textWidth,"L/sqm");
					else
						if oldValue == nil then print(fruitName.." : old literPerSqm not defined");end;
						if myValue  == nil then print(fruitName.." : my literPerSqm not defined");end;
					end;
				else
					print("xmlKey 3 does not have xml property");
				end;

				local xmlKey4 = string.format("map.fruitTypes.fruitType(%d).windrow", i);
				if hasXMLProperty(xmlFile, xmlKey4) then
					local oldValue = g_fruitTypeManager.nameToFruitType[fruitName].windrowLiterPerSqm;
					local myValue = getXMLFloat(xmlFile, xmlKey4.. "#litersPerSqm");
					if oldValue ~= nil and myValue ~= nil then
						g_fruitTypeManager.nameToFruitType[fruitName].windrowLiterPerSqm = myValue;
						myPrintScaledValue(fruitName.." windrow litersPerSqm",oldValue,myValue,textWidth,"L/sqm");
					else
						if oldValue == nil then print(fruitName.." : old windrow literPerSqm not defined");end;
						if myValue  == nil then print(fruitName.." : my windrow literPerSqm not defined");end;
					end;
--					else
--						print(fruitName.." does not have windrow");
				end;
				print(" ");
			else
				print(" ");
				print("Not in map : "..fruitName);
				print(" ");
			end; -- if fruit is in map
			i = i + 1;
		end;	-- while

	else

		print("\n --- Simple one-factor parameter scaling in effect --- \n")
		for i, v in pairs(g_currentMission.fruitTypeManager.fruitTypes) do

			local fruitName = tostring(g_currentMission.fruitTypeManager.fruitTypes[i].name);
--			print(fruitName);

			if g_currentMission.fruitTypeManager.fruitTypes[i].seedUsagePerSqm  ~= nil then
				local oldValue = g_currentMission.fruitTypeManager.fruitTypes[i].seedUsagePerSqm;
				local myValue = oldValue * RN.fruitTypes.seedUsageScale;

				if oldValue ~= nil and myValue ~= nil then
					g_currentMission.fruitTypeManager.fruitTypes[i].seedUsagePerSqm  = myValue;
					myPrintScaledValue(fruitName.." seedUsagePerSqm",oldValue,myValue,textWidth,"L/sqm");
				else
					if oldValue == nil then print(fruitName.." : old seedUsagePerSqm not defined");end;
					if myValue  == nil then print(fruitName.." : my seedUsagePerSqm not defined");end;
				end;																									
			end;	
			
			if g_currentMission.fruitTypeManager.fruitTypes[i].literPerSqm ~= nil then			
				local oldValue = g_currentMission.fruitTypeManager.fruitTypes[i].literPerSqm;
				local myValue = oldValue * RN.fruitTypes.yieldScale;
														
				if oldValue ~= nil and myValue ~= nil then
					g_currentMission.fruitTypeManager.fruitTypes[i].literPerSqm = myValue;
					myPrintScaledValue(fruitName.." yield literPerSqm",oldValue,myValue,textWidth,"L/sqm");
				else
					if oldValue == nil then print(fruitName.." : old literPerSqm not defined");end;
					if myValue  == nil then print(fruitName.." : my literPerSqm not defined");end;
				end;
			end;
			
			if g_currentMission.fruitTypeManager.fruitTypes[i].windrowLiterPerSqm ~= nil then
				local oldValue = g_currentMission.fruitTypeManager.fruitTypes[i].windrowLiterPerSqm;
				local myValue = oldValue * RN.fruitTypes.yieldScaleWindrow;

				if oldValue ~= nil and myValue ~= nil then
					g_currentMission.fruitTypeManager.fruitTypes[i].windrowLiterPerSqm  = myValue;
					myPrintScaledValue(fruitName.." windrow litersPerSqm",oldValue,myValue,textWidth,"L/sqm");
				else
					if oldValue == nil then print(fruitName.." : old windrow litersPerSqm not defined");end;
					if myValue  == nil then print(fruitName.." : my windrow literPerSqm not defined");end;
				end;																									
			end;
			print(" ");
		end;

	end;

		print("\n --- RealNumbersFruitTypes.lua finished --- \n");

end;


function RealNumbersFruitTypes:keyEvent(unicode, sym, modifier, isDown)
end;
function RealNumbersFruitTypes:update(dt)
end;
function RealNumbersFruitTypes:draw()
end;
function RealNumbersFruitTypes:deleteMap()
end;
function RealNumbersFruitTypes:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersFruitTypes);







