--
-- Mod: FS22_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersLoans by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.0.1.0 
-- Version 1.0.1.1 - no change
-- Version 1.0.1.2 - no change
-- Version 1.0.1.3 - no change
-- Version 1.0.1.4 - no change
-- Version 1.0.1.5 - no change
-- Version 1.0.1.6 - no change
-- Version 1.0.1.7 - no change
-- Version 1.0.1.8 - no change
-- Version 1.0.1.9 - no change
-- Version 1.0.2.0 - no change
-- Version 1.0.2.1 - no change
-- Version 1.0.2.2 - Disabled in modDesc.xml Use the mod EnhancedLoanSystem

--
-- Objective
--
-- This script reads the data in the xml file my_Loans.xml located in the folder
-- modSettings/RealLifeNumbers/savegame(n) and save the numbers into the game.
-- 
-- This way you can use different parameter values for different maps and adjust the 
-- values to fit values more suitable to a specific region/state/country.


RealNumbersLoans = {};

function myPrint(text, number, textWidth, unit)
	print(string.format("%s =  %8.0f %s", text..string.rep(" ",textWidth-string.len(text)),number,"    "..unit));
end;

function RealNumbersLoans:loadMap(name)

	local currency = "€";	-- or  "$" or "£"
	local gameMoneyUnit = g_i18n.moneyUnit;
	if gameMoneyUnit == 1 then
        currency = "€";
	elseif gameMoneyUnit == 2 then
        currency = "$";
	elseif gameMoneyUnit == 3 then
        currency = "£";
	end;

	local useMyXMLfile = true;
	local textWidth = 40;
	print("\n --- RealNumbersLoans.lua started --- \n");
	RN.notRunYetRealNumbersLoans = true;
	RN.upStartRealNumbersLoans = true;
	RN.farms = {};
	RN.farms.loanAnnualInterestRate = 4;
	RN.farms.loanMax = 2000000;

	local modSettingsDir = getUserProfileAppPath() .. "modSettings";	
	if modSettingsDir == nil then
		print(" ");
		print("modSettingsDir undefined");
		print(" ");
	end;

	local savegameIndex = 0;
	if g_currentMission.missionInfo.savegameIndex ~= nil then
		savegameIndex = g_currentMission.missionInfo.savegameIndex;
	end;
	local RNmodSettingsDir = modSettingsDir .. "/RealLifeNumbers/savegame" .. savegameIndex;
	
	local myLoansXML = RNmodSettingsDir.."/my_Loans.xml";


	if fileExists(myLoansXML) then
		print("\n --- my XML file is used --- \n")
		local xmlFile = loadXMLFile("loans", myLoansXML);
		print("Loading file:");
		print(myLoansXML);
		print(" ");

		local i = 0;
		while true do

			local xmlKey = string.format("loans.farm(%d)", i);
			if not hasXMLProperty(xmlFile, xmlKey) then
--			print(" end of file or does not have xml property");
				break;
			end;

			local loanAnnualInterestRate = tonumber(getXMLString(xmlFile, xmlKey .. "#loanAnnualInterestRate"));
			local loanMax = tonumber(getXMLString(xmlFile, xmlKey.. "#loanMax"));
			RN.farms.loanAnnualInterestRate = loanAnnualInterestRate;
			RN.farms.loanMax = loanMax;
			print(" ");
			myPrint("loanAnnualInterestRate",RN.farms.loanAnnualInterestRate,textWidth," ");
			myPrint("Max loan",RN.farms.loanMax,textWidth,currency);
			i = i + 1;
		end;	
	else
		print("xml file does not exist for this savegame. Default parameters are used.");
	end;
	print("\n --- RealNumbersLoans.lua updates every 10 minutes --- \n");

end;


function RealNumbersLoans:keyEvent(unicode, sym, modifier, isDown)
end;

function RealNumbersLoans:update(dt)
	local minute = g_currentMission.environment.currentMinute;
	local runMinute = minute % 10; -- remainder
	if runMinute ~= 8 then RN.notRunYetRealNumbersLoans = true; end;
	if runMinute == 8 or RN.upStartRealNumbersLoans then
		if RN.notRunYetRealNumbersLoans then

			if g_farmManager.farms ~= nil then
				for k,farm in pairs(g_farmManager.farms) do
						Farm.LOAN_INTEREST_RATE = RN.farms.loanAnnualInterestRate/100
						farm.loanMax = RN.farms.loanMax;
				end;
			else
				print("no farms in RealNumbersLoan:update");
			end;
		end;
		RN.notRunYetRealNumbersLoans = false;
		RN.upStartRealNumbersLoans = false;
	end;
end;
function RealNumbersLoans:draw()
end;
function RealNumbersLoans:deleteMap()
end;
function RealNumbersLoans:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersLoans);






