--
-- Mod: FS22_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersSellingStations by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.0.1.0 
-- Version 1.0.1.1 - no change
-- Version 1.0.1.2 - no change
-- Version 1.0.1.3 - no change
-- Version 1.0.1.4 - no change
-- Version 1.0.1.5 - no change
-- Version 1.0.1.6 - no change
-- Version 1.0.1.7 - no change
-- Version 1.0.1.8 - no change
-- Version 1.0.1.9 - no change
-- Version 1.0.2.0 - no change
-- Version 1.0.2.1 - no change
-- Version 1.0.2.2 - no change
-- Version 1.0.2.3 - no change

RealNumbersSellingStations = {};

local function print_r (t, indent, done)
  done = done or {}
  indent = indent or ''
  local nextIndent -- Storage for next indentation value
  for key, value in pairs (t) do
    if type (value) == "table" and not done [value] then
      nextIndent = nextIndent or
          (indent .. string.rep(' ',string.len(tostring (key))+2))
          -- Shortcut conditional allocation
      done [value] = true
      print (indent .. "[" .. tostring (key) .. "] => Table {");
      print  (nextIndent .. "{");
      print_r (value, nextIndent .. string.rep(' ',2), done)
      print  (nextIndent .. "}");
    else
      print  (indent .. "[" .. tostring (key) .. "] => " .. tostring (value).."")
    end
  end
end

function RealNumbersSellingStations:loadMap(name)
	print("\n --- RealNumbersSellingStations started  --- \n")
	print("\n --- Reduces the difference between lowest and highest sell price --- \n");
	RN.notRunYetRealNumbersSellingStations = true;
	RN.upStartRealNumbersSellingStations = true;
	
	RN.sellingStations = {};
	RN.sellingStations.amplitudeScale 					= 0.05;		-- default RLN value = 0.05
	RN.sellingStations.nominalAmplitudeScale 			= 0.05;		-- default RLN value = 0.05	
	RN.sellingStations.nominalAmplitudeVariationScale 	= 0.005;	-- default RLN value = 0.005 
	
-- These parameter values produce about 10-20% price variation between sellingstations 
-- The exact algorith is unknown. Trial and error is the only approach

end;

function RealNumbersSellingStations:keyEvent(unicode, sym, modifier, isDown)
end;

function RealNumbersSellingStations:update(dt)
	local minute = g_currentMission.environment.currentMinute;
	local runMinute = minute % 10; -- remainder
	if runMinute ~= 9 then RN.notRunYetRealNumbersSellingStations = true; end;
	if runMinute == 9 or RN.upStartRealNumbersSellingStations then 
		if RN.notRunYetRealNumbersSellingStations then
--			print("Updating RealNumbersSellingStations"); -- debug
			for k,placeable in pairs(g_currentMission.placeableSystem.placeables) do
--				print(k);	-- debug
				local name = placeable.typeName;
--				print(name); 	-- debug
				if name == "sellingStation" then
					local sellingStation = placeable.spec_sellingStation.sellingStation;
					local isSellingStation = sellingStation.isSellingPoint;
--					if isSellingStation then print("This is a sellingstation"); end;	-- debug
					if sellingStation ~= nil then
						for kk, fillTypeCurve in pairs(sellingStation.pricingDynamics) do
--							print(k); -- debug
--							print_r(fillTypeCurve);	-- debug
							local originalFillTypePricesUnscaled = sellingStation.originalFillTypePricesUnscaled;
							local fillName = tostring(g_fillTypeManager:getFillTypeNameByIndex(kk));
--							print(fillName);	-- debug
--							print(originalFillTypePricesUnscaled[kk]);	-- debug
							fillTypeCurve.curves[1].amplitude = RN.sellingStations.amplitudeScale * originalFillTypePricesUnscaled[kk];
							fillTypeCurve.curves[1].nominalAmplitude  = RN.sellingStations.nominalAmplitudeScale * originalFillTypePricesUnscaled[kk];
							fillTypeCurve.curves[1].nominalAmplitudeVariation  = RN.sellingStations.nominalAmplitudeVariationScale * originalFillTypePricesUnscaled[kk];
							
							fillTypeCurve.baseCurve.amplitude = RN.sellingStations.amplitudeScale * originalFillTypePricesUnscaled[kk];
							fillTypeCurve.baseCurve.nominalAmplitude  = RN.sellingStations.nominalAmplitudeScale * originalFillTypePricesUnscaled[kk];
							fillTypeCurve.baseCurve.nominalAmplitudeVariation  = RN.sellingStations.nominalAmplitudeVariationScale * originalFillTypePricesUnscaled[kk];
						end;
					end;
				end;
			end;
		end;	
		RN.notRunYetRealNumbersSellingStations = false;
		RN.upStartRealNumbersSellingStations = false;
	end;
end;
function RealNumbersSellingStations:draw()
end;
function RealNumbersSellingStations:deleteMap()
end;
function RealNumbersSellingStations:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersSellingStations);