--
-- Mod: FS22_RealLifeNumbers by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- Script: RealNumbersSprayTypes by Kaj-Aage Henneberg "Ki", ArmChairFarming
-- e-mail: kaj-aage.henneberg@outlook.dk
-- Download: www.modhoster.com
-- All rights reserved.
-- Version 1.0.0.7 
-- 26-11-2021
-- Version 1.0.0.8 - Adjusted spray rates
-- Version 1.0.0.9 - Nothing new
-- Version 1.0.1.0 - Added in-script values to use if xml file is not used.
-- Version 1.0.1.1 - no change
-- Version 1.0.1.2 - no change
-- Version 1.0.1.3 - no change
-- Version 1.0.1.4 - no change
-- Version 1.0.1.5 - no change
-- Version 1.0.1.6 - no change
-- Version 1.0.1.7 - no change
-- Version 1.0.1.8 - no change
-- Version 1.0.1.9 - no change
-- Version 1.0.2.0 - no change
-- Version 1.0.2.1 - no change
-- Version 1.0.2.2 - Disabled in modDesc. Use the mod precisionFarming instead.

-- Objective
--
-- This script reads the data in the xml file my_sprayTypes.xml located in the folder
-- modSettings/RealLifeNumbers/savegame(n) and save the numbers into the game.
-- 
-- This way you can use different parameter values for different maps and adjust the 
-- values to fit values more suitable to a specific region/state/country.

RealNumbersSprayTypes = {};

local function myPrintScaledValue(text, number1, number2,textWidth, unit)
	print(string.format("%s : default value =  %12.5f : my value = %12.5f %s", text..string.rep(" ",textWidth-string.len(text)),number1, number2,"    "..unit));
end;

function RealNumbersSprayTypes:loadMap(name)
	local useMyXMLfile = true;
	local textWidth = 40;
	print("\n --- RealNumbersSprayTypes.lua started --- \n");

	RN.sprayTypes = {};

	local modSettingsDir = getUserProfileAppPath() .. "modSettings";	
	if modSettingsDir == nil then
		print(" ");
		print("modSettingsDir undefined");
		print(" ");
	end;
	local savegameIndex = 0;
	if g_currentMission.missionInfo.savegameIndex ~= nil then
		savegameIndex = g_currentMission.missionInfo.savegameIndex;
	end;
	local RNmodSettingsDir = modSettingsDir .. "/RealLifeNumbers/savegame" .. savegameIndex;
	
	local mySprayTypeXML = RNmodSettingsDir.."/my_sprayTypes.xml";


	if fileExists(mySprayTypeXML) then
		print("\n --- my XML file is used --- \n")
		local xmlFile = loadXMLFile("sprayTypes", mySprayTypeXML);
		print("Loading file:");
		print(mySprayTypeXML);
		print(" ");

		local i = 0;
		while true do
			local xmlKey = string.format("map.sprayTypes.sprayType(%d)", i);
			if not hasXMLProperty(xmlFile, xmlKey) then
--			print(" does not have xml property");
				break;
			end;
				local sprayName = getXMLString(xmlFile, xmlKey .. "#name");	
				local oldValue = g_sprayTypeManager.nameToSprayType[sprayName].litersPerSecond;
				local myValue = getXMLFloat(xmlFile, xmlKey.. "#litersPerSecond");
				g_sprayTypeManager.nameToSprayType[sprayName].litersPerSecond = myValue;
				myPrintScaledValue(sprayName.." litersPerSecond",oldValue,myValue,textWidth," ");
			i = i + 1;
		end;	
	else
		print("\n --- xml file does not exist for this savegame. In-script parameters are used. --- \n");
		useMyXMLfile = false;
		g_sprayTypeManager.nameToSprayType["DIGESTATE"].litersPerSecond 		= 0.8333;	-- 30000L/ha  = 18 Tonnes/ha
		g_sprayTypeManager.nameToSprayType["FERTILIZER"].litersPerSecond 		= 0.0139;	-- 500L/ha
		g_sprayTypeManager.nameToSprayType["HERBICIDE"].litersPerSecond 		= 0.0035;	-- 125L/ha
		g_sprayTypeManager.nameToSprayType["LIME"].litersPerSecond 				= 0.1301;	-- 4700L/ha
		g_sprayTypeManager.nameToSprayType["LIQUIDFERTILIZER"].litersPerSecond 	= 0.0069;	-- 250L/ha
		g_sprayTypeManager.nameToSprayType["LIQUIDMANURE"].litersPerSecond 		= 0.8333;	-- 30000L/ha  = 18 Tonnes/ha
		g_sprayTypeManager.nameToSprayType["MANURE"].litersPerSecond 			= 0.8333;	-- 30000L/ha  = 27 Tonnes/ha
--		createFolder(RNmodSettingsDir);
	end;
	print("\n --- RealNumbersSprayTypes.lua finished --- \n");
end;


function RealNumbersSprayTypes:keyEvent(unicode, sym, modifier, isDown)
end;
function RealNumbersSprayTypes:update(dt)
end;
function RealNumbersSprayTypes:draw()
end;
function RealNumbersSprayTypes:deleteMap()
end;
function RealNumbersSprayTypes:mouseEvent(posX, posY, isDown, isUp, button)
end;

addModEventListener(RealNumbersSprayTypes);






